#include "testconfig.h"
#ifdef HAVE_FMA
#define TEST_FUNC_FFF_F __MATH_PRECNAME(fma)
#include "testdriver.h"


#define ONE_P HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L)
#define ONE_M HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L)

#define LDBL_HALF_MAX_P HEXCONSTE(5.9486574767861588251063e+4931, 0x7ffd, 0xffffffffL, 0xffffffffL)
#define LDBL_HALF_MAX_M HEXCONSTE(-5.9486574767861588251063e+4931, 0xfffd, 0xffffffffL, 0xffffffffL)
#define LDBL_4TH_MAX_M HEXCONSTE(-2.9743287383930794125532e+4931, 0xfffc, 0xffffffffL, 0xffffffffL)


static test_fff_f_data const fma_data[] = {
	{ __LINE__, ONE_P, HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(5.0, 0x4001, 0xa0000000L, 0x00000000L), 0 },
	{ __LINE__, QNAN_P, HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, ONE_P, QNAN_P, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, ONE_P, HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	{ __LINE__, INF_P, ZERO_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	{ __LINE__, INF_M, ZERO_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	{ __LINE__, ZERO_P, INF_P, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	{ __LINE__, ZERO_P, INF_M, QNAN_P, QNAN_P, INVALID_EXCEPTION_OK },
	{ __LINE__, INF_P, ZERO_P, ONE_P, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_M, ZERO_P, ONE_P, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, ZERO_P, INF_P, ONE_P, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, ZERO_P, INF_M, ONE_P, QNAN_P, INVALID_EXCEPTION },

	{ __LINE__, INF_P, INF_P, INF_M, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_M, INF_P, INF_P, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_P, INF_M, INF_P, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_M, INF_M, INF_M, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_P, HEXCONSTE(3.5L, 0x4000, 0xe0000000L, 0x00000000L), INF_M, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_M, HEXCONSTE(-7.5L, 0xc001, 0xf0000000L, 0x00000000L), INF_M, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, HEXCONSTE(-13.5L, 0xc002, 0xd8000000L, 0x00000000L), INF_P, INF_P, QNAN_P, INVALID_EXCEPTION },
	{ __LINE__, INF_M, HEXCONSTE(7.5L, 0x4001, 0xf0000000L, 0x00000000L), INF_P, QNAN_P, INVALID_EXCEPTION },

	{ __LINE__, HEXCONSTE(1.25L, 0x3fff, 0xa0000000L, 0x00000000L), HEXCONSTE(0.75L, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(0.0625L, 0x3ffb, 0x80000000L, 0x00000000L), ONE_P, 0 },

	/* If x times y is an exact infinity, and z is an infinity with the opposite sign, a domain error occurs, and a NaN is returned.*/
	{ __LINE__, MAX_M, MAX_M, INF_M, QNAN_P, 0 },
	{ __LINE__, LDBL_HALF_MAX_P, LDBL_HALF_MAX_P, INF_M, QNAN_P, 0 },
	{ __LINE__, MAX_M, MAX_P, INF_P, QNAN_P, 0 },
	{ __LINE__, LDBL_HALF_MAX_P, LDBL_4TH_MAX_M, INF_P, QNAN_M, 0 },
#ifdef TEST_LDOUBLE
	{ __LINE__, INF_P, HEXCONSTE(4, 0x4001, 0x80000000L, 0x00000000L), INF_P, QNAN_M, 0 },
	{ __LINE__, HEXCONSTE(2, 0x4000, 0x80000000L, 0x00000000L), INF_M, INF_M, QNAN_M, 0 },
	{ __LINE__, INF_M, INF_M, INF_P, QNAN_M, 0 },
	{ __LINE__, INF_P, INF_M, INF_M, QNAN_M, 0 },
#endif

	{ __LINE__, ZERO_P, ZERO_P, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ZERO_P, ZERO_M, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ZERO_M, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ZERO_M, ZERO_M, ZERO_M, 0 },
	{ __LINE__, ZERO_M, ZERO_P, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_M, ZERO_P, ZERO_M, ZERO_M, 0 },
	{ __LINE__, ZERO_M, ZERO_M, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_M, ZERO_M, ZERO_M, ZERO_P, 0 },
	{ __LINE__, ONE_P, ZERO_P, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ONE_P, ZERO_P, ZERO_M, ZERO_P, 0 },
	{ __LINE__, ONE_P, ZERO_M, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ONE_P, ZERO_M, ZERO_M, ZERO_M, 0 },
	{ __LINE__, ONE_M, ZERO_P, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ONE_M, ZERO_P, ZERO_M, ZERO_M, 0 },
	{ __LINE__, ONE_M, ZERO_M, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ONE_M, ZERO_M, ZERO_M, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ONE_P, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ONE_P, ZERO_M, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ONE_M, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_P, ONE_M, ZERO_M, ZERO_M, 0 },
	{ __LINE__, ZERO_M, ONE_P, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_M, ONE_P, ZERO_M, ZERO_M, 0 },
	{ __LINE__, ZERO_M, ONE_M, ZERO_P, ZERO_P, 0 },
	{ __LINE__, ZERO_M, ONE_M, ZERO_M, ZERO_P, 0 },

	{ __LINE__, ONE_P, ONE_P, ONE_M, ZERO_P, 0 },
	{ __LINE__, ONE_P, ONE_M, ONE_P, ZERO_P, 0 },
	{ __LINE__, ONE_M, ONE_P, ONE_P, ZERO_P, 0 },
	{ __LINE__, ONE_M, ONE_M, ONE_M, ZERO_P, 0 },

	{ __LINE__, MIN_P, MIN_P, ZERO_P, ZERO_P, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_P, MIN_P, ZERO_M, ZERO_P, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_P, MIN_M, ZERO_P, ZERO_M, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_P, MIN_M, ZERO_M, ZERO_M, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_M, MIN_P, ZERO_P, ZERO_M, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_M, MIN_P, ZERO_M, ZERO_M, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_M, MIN_M, ZERO_P, ZERO_P, UNDERFLOW_EXCEPTION },
	{ __LINE__, MIN_M, MIN_M, ZERO_M, ZERO_P, UNDERFLOW_EXCEPTION },

	{ __LINE__, MAX_P, MAX_P, MIN_P, INF_P, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_P, MAX_P, MIN_M, INF_P, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_P, MAX_M, MIN_P, INF_M, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_P, MAX_M, MIN_M, INF_M, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_M, MAX_P, MIN_P, INF_M, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_M, MAX_P, MIN_M, INF_M, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_M, MAX_M, MIN_P, INF_P, OVERFLOW_EXCEPTION },
	{ __LINE__, MAX_M, MAX_M, MIN_M, INF_P, OVERFLOW_EXCEPTION },

#if defined (TEST_FLOAT) && FLT_MANT_DIG == 24
	{ __LINE__, HEXCONSTE(1.22870000000000000000e+04, 0x400c, 0xbffc0000L, 0x00000000L), HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.19208380056079477072e-07, 0x3fe7, 0xffff8000L, 0x00000000L), HEXCONSTE(1.22870009765625000000e+04, 0x400c, 0xbffc0100L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.99975585937500000000e+00, 0x3fff, 0xfff80000L, 0x00000000L), HEXCONSTE(1.00000095367431640625e+00, 0x3fff, 0x80000800L, 0x00000000L), HEXCONSTE(-1.99975585937500000000e+00, 0xbfff, 0xfff80000L, 0x00000000L), HEXCONSTE(1.90711580216884613037e-06, 0x3feb, 0xfff80000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.72982066388076033078e+38, 0x407e, 0xcd5e6f00L, 0x00000000L), HEXCONSTE(7.10520959454037574281e-39, 0x3f80, 0x9abcde00L, 0x00000000L), HEXCONSTE(-1.93959474563598632812e+00, 0xbfff, 0xf844a400L, 0x00000000L), HEXCONSTE(5.16020293162000598386e-08, 0x3fe6, 0xdda10800L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.03387488713917160837e+30, 0x4063, 0xcd5e6f00L, 0x00000000L), HEXCONSTE(7.10520959454037574281e-39, 0x3f80, 0x9abcde00L, 0x00000000L), HEXCONSTE(-1.44511069777308875928e-08, 0xbfe4, 0xf844a400L, 0x00000000L), HEXCONSTE(3.84465078385174720277e-16, 0x3fcb, 0xdda10800L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00024414062500000000e+00, 0x3fff, 0x80080000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(8.30767447847970849150e+34, 0x4072, 0xffffff00L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.99999988079071044922e+00, 0x3fff, 0xffffff00L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.40282306073709652508e+38, 0xc07e, 0xfffffd00L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.46936793852785938496e-38, 0x3f81, 0xa0000000L, 0x00000000L), HEXCONSTE(5.00000119209289550781e-01, 0x3ffe, 0x80000200L, 0x00000000L), HEXCONSTE(2.93873587705571876992e-39, 0x3f7f, 0x80000000L, 0x00000000L), HEXCONSTE(1.02855769709934800195e-38, 0x3f80, 0xe0000200L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.46936793852785938496e-38, 0xbf81, 0xa0000000L, 0x00000000L), HEXCONSTE(5.00000119209289550781e-01, 0x3ffe, 0x80000200L, 0x00000000L), HEXCONSTE(-2.93873587705571876992e-39, 0xbf7f, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02855769709934800195e-38, 0xbf80, 0xe0000200L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(2.35098814112518928601e-38, 0x3f81, 0xfffffc00L, 0x00000000L), HEXCONSTE(5.00000059604644775391e-01, 0x3ffe, 0x80000100L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(-2.35098814112518928601e-38, 0xbf81, 0xfffffc00L, 0x00000000L), HEXCONSTE(5.00000059604644775391e-01, 0x3ffe, 0x80000100L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549421069244107549e-38, 0xbf80, 0xfffffe00L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(5.31250000000000000000e-01, 0x3ffe, 0x88000000L, 0x00000000L), HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(5.31250000000000000000e-01, 0x3ffe, 0x88000000L, 0x00000000L), HEXCONSTE(-1.17549421069244107549e-38, 0xbf80, 0xfffffe00L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549421069244107549e-38, 0xbf80, 0xfffffe00L, 0x00000000L), HEXCONSTE(-1.17549421069244107549e-38, 0xbf80, 0xfffffe00L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549421069244107549e-38, 0xbf80, 0xfffffe00L, 0x00000000L), HEXCONSTE(-1.17549421069244107549e-38, 0xbf80, 0xfffffe00L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(9.99755859375000000000e-01, 0x3ffe, 0xfff00000L, 0x00000000L), HEXCONSTE(9.99755859375000000000e-01, 0x3ffe, 0xfff00000L, 0x00000000L), HEXCONSTE(-9.99511718750000000000e-01, 0xbffe, 0xffe00000L, 0x00000000L), HEXCONSTE(5.96046447753906250000e-08, 0x3fe7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(9.99755859375000000000e-01, 0x3ffe, 0xfff00000L, 0x00000000L), HEXCONSTE(-9.99755859375000000000e-01, 0xbffe, 0xfff00000L, 0x00000000L), HEXCONSTE(9.99511718750000000000e-01, 0x3ffe, 0xffe00000L, 0x00000000L), HEXCONSTE(-5.96046447753906250000e-08, 0xbfe7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99755859375000000000e-01, 0xbffe, 0xfff00000L, 0x00000000L), HEXCONSTE(9.99755859375000000000e-01, 0x3ffe, 0xfff00000L, 0x00000000L), HEXCONSTE(9.99511718750000000000e-01, 0x3ffe, 0xffe00000L, 0x00000000L), HEXCONSTE(-5.96046447753906250000e-08, 0xbfe7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99755859375000000000e-01, 0xbffe, 0xfff00000L, 0x00000000L), HEXCONSTE(-9.99755859375000000000e-01, 0xbffe, 0xfff00000L, 0x00000000L), HEXCONSTE(-9.99511718750000000000e-01, 0xbffe, 0xffe00000L, 0x00000000L), HEXCONSTE(5.96046447753906250000e-08, 0x3fe7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(1.49011629702044956503e-08, 0x3fe5, 0x80000100L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(-1.49011629702044956503e-08, 0xbfe5, 0x80000100L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(1.49011629702044956503e-08, 0x3fe5, 0x80000100L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(-1.49011629702044956503e-08, 0xbfe5, 0x80000100L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(1.49011629702044956503e-08, 0x3fe5, 0x80000100L, 0x00000000L), HEXCONSTE(1.01412048018258352120e+31, 0x4066, 0x80000000L, 0x00000000L), HEXCONSTE(1.01412048018258352120e+31, 0x4066, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(-1.49011629702044956503e-08, 0xbfe5, 0x80000100L, 0x00000000L), HEXCONSTE(1.01412048018258352120e+31, 0x4066, 0x80000000L, 0x00000000L), HEXCONSTE(1.01412048018258352120e+31, 0x4066, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(1.49011629702044956503e-08, 0x3fe5, 0x80000100L, 0x00000000L), HEXCONSTE(-1.01412048018258352120e+31, 0xc066, 0x80000000L, 0x00000000L), HEXCONSTE(-1.01412048018258352120e+31, 0xc066, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549449095213394045e-38, 0x3f81, 0x80000100L, 0x00000000L), HEXCONSTE(-1.49011629702044956503e-08, 0xbfe5, 0x80000100L, 0x00000000L), HEXCONSTE(-1.01412048018258352120e+31, 0xc066, 0x80000000L, 0x00000000L), HEXCONSTE(-1.01412048018258352120e+31, 0xc066, 0x80000000L, 0x00000000L), 0 },
#endif
#if defined (TEST_DOUBLE) && DBL_MANT_DIG == 53
	{ __LINE__, HEXCONSTE(1.22560000000000000000e+04, 0x400c, 0xbf800000L, 0x00000000L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(7.10369263412502505162e-15, 0x3fcf, 0xfff00000L, 0x00000000L), HEXCONSTE(1.22560000000000018190e+04, 0x400c, 0xbf800000L, 0x00000800L), 0 },
	{ __LINE__, HEXCONSTE(1.99975585937500000000e+00, 0x3fff, 0xfff80000L, 0x00000000L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-1.99975585937500000000e+00, 0xbfff, 0xfff80000L, 0x00000000L), HEXCONSTE(4.44034999741438340948e-16, 0x3fcb, 0xfff80000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000745058059692e+00, 0x3fff, 0x80000010L, 0x00000000L), HEXCONSTE(9.99999992549419403076e-01, 0x3ffe, 0xffffffe0L, 0x00000000L), HEXCONSTE(4.90909346529772655310e-91, 0x3ed3, 0x80000000L, 0x00000000L), HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000745058059692e+00, 0x3fff, 0x80000010L, 0x00000000L), HEXCONSTE(9.99999992549419403076e-01, 0x3ffe, 0xffffffe0L, 0x00000000L), HEXCONSTE(-4.90909346529772655310e-91, 0xbed3, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), 0 },
	{ __LINE__, HEXCONSTE(1.68069805317375865044e+308, 0x43fe, 0xef56df77L, 0x97f76800L), HEXCONSTE(1.93545524363825017444e-308, 0x3c00, 0xdeadbeefL, 0x2feed000L), HEXCONSTE(-3.25291585998774968047e+00, 0xc000, 0xd02fc600L, 0xd25fd800L), HEXCONSTE(1.15158208725138169733e-16, 0x3fca, 0x84c4b43dL, 0xe4ed2000L), 0 },
	{ __LINE__, HEXCONSTE(1.58052085355501590052e+271, 0x4383, 0xef56df77L, 0x97f76800L), HEXCONSTE(1.93545524363825017444e-308, 0x3c00, 0xdeadbeefL, 0x2feed000L), HEXCONSTE(-3.05902737369265831580e-37, 0xbf85, 0xd02fc600L, 0xd25fd800L), HEXCONSTE(1.08294259045771022372e-53, 0x3f4f, 0x84c4b43dL, 0xe4ed2000L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00024414062500000000e+00, 0x3fff, 0x80080000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.38889925503495045934e+304, 0x43f2, 0xffffffffL, 0xfffff800L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.99999999999999977796e+00, 0x3fff, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.79769313486231530898e+308, 0xc3fe, 0xffffffffL, 0xffffe800L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), 0 },
	{ __LINE__, HEXCONSTE(1.57172778470262854957e-162, 0x3de5, 0xb504f333L, 0xf9de6000L), HEXCONSTE(1.57172778470262854957e-162, 0x3de5, 0xb504f333L, 0xf9de6000L), ZERO_P, ZERO_P, UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.82791618850005775053e-149, 0x3e10, 0xef56df77L, 0x97f76800L), HEXCONSTE(1.82791618850005775053e-149, 0x3e10, 0xef56df77L, 0x97f76800L), HEXCONSTE(-3.34127759218057860347e-298, 0xbc22, 0xdfc352bcL, 0x352ba000L), HEXCONSTE(5.50261588891002961624e-315, 0x3beb, 0x84c4b43eL, 0x00000000L), UNDERFLOW_EXCEPTION|FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(7.14029761132835058799e-152, 0x3e08, 0xef56df77L, 0x97f76800L), HEXCONSTE(7.14029761132835058799e-152, 0x3e08, 0xef56df77L, 0x97f76800L), HEXCONSTE(-5.09838499783413483195e-303, 0xbc12, 0xdfc352bcL, 0x352ba000L), HEXCONSTE(8.39615158542614377174e-320, 0x3bdb, 0x84c40000L, 0x00000000L), UNDERFLOW_EXCEPTION|FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(2.22275874948507748344e-162, 0x3de6, 0x80000000L, 0x00000000L), HEXCONSTE(1.11137937474253874172e-162, 0x3de5, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(9.88131291682493088353e-324, 0x3bce, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(6.01250496056917516558e-292, 0x3c37, 0xbfffffc0L, 0x00000000L), HEXCONSTE(1.23259516440783094596e-32, 0x3f95, 0x80000000L, 0x00000000L), HEXCONSTE(1.32624736935329519694e-315, 0x3be9, 0x80000000L, 0x00000000L), HEXCONSTE(1.32624737429395165536e-315, 0x3be9, 0x80000008L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.00208419195026070584e-291, 0x3c38, 0xa0000020L, 0x00000000L), HEXCONSTE(1.23259516440783094596e-32, 0x3f95, 0x80000000L, 0x00000000L), HEXCONSTE(1.32624736935329519694e-315, 0x3be9, 0x80000000L, 0x00000000L), HEXCONSTE(1.32624738417526457218e-315, 0x3be9, 0x80000018L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.00208418000448638900e-291, 0x3c38, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.23259516440783094596e-32, 0xbf95, 0x80000000L, 0x00000000L), HEXCONSTE(-1.32624736935329519694e-315, 0xbbe9, 0x80000000L, 0x00000000L), HEXCONSTE(-1.32624737923460811377e-315, 0xbbe9, 0x80000010L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-2.25903320843691287941e-289, 0xbc40, 0x8ce55b36L, 0xb9f0b800L), HEXCONSTE(1.09552820860933426296e-32, 0x3f94, 0xe3884546L, 0x2ffa8800L), HEXCONSTE(-1.11853546624563400596e-308, 0xbc00, 0x80b0ad65L, 0xd9b64000L), HEXCONSTE(-1.11853546624588153284e-308, 0xbc00, 0x80b0ad65L, 0xd9d59000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-3.56972370854509549663e-295, 0xbc2c, 0xe97576b7L, 0x474e9800L), HEXCONSTE(-1.41465128783349729973e-19, 0xbfc0, 0xa7033631L, 0x564ee800L), HEXCONSTE(-1.27136071315873422165e-308, 0xbc00, 0x9245e6b0L, 0x03454000L), HEXCONSTE(-1.27135566324449266374e-308, 0xbc00, 0x9245c09cL, 0x5fb5d000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(1.00094353880352101309e-273, 0x3c74, 0x8a9eb285L, 0xdcf83000L), HEXCONSTE(2.76428921941130553870e-38, 0x3f82, 0x96809186L, 0xa4203800L), HEXCONSTE(-1.55122423784859904827e-308, 0xbc00, 0xb278d5acL, 0xfc3c0000L), HEXCONSTE(-1.54845734041504526743e-308, 0xbc00, 0xb2275712L, 0x3bbe9000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-1.85653473271011680903e-214, 0xbd38, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.29443685725953814185e-83, 0x3eec, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.44649443426719370386e-296, 0x3c28, 0xffffff00L, 0x003ff800L), HEXCONSTE(1.83487078924492417933e-296, 0x3c28, 0xbfffff00L, 0x003ff800L), 0 },
	{ __LINE__, HEXCONSTE(2.78134232313400172886e-308, 0x3c01, 0xa0000000L, 0x00000000L), HEXCONSTE(5.00000000000000222045e-01, 0x3ffe, 0x80000000L, 0x00001000L), HEXCONSTE(5.56268464626800345773e-309, 0x3bff, 0x80000000L, 0x00000000L), HEXCONSTE(1.94693962619380170427e-308, 0x3c00, 0xe0000000L, 0x00001000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-2.78134232313400172886e-308, 0xbc01, 0xa0000000L, 0x00000000L), HEXCONSTE(5.00000000000000222045e-01, 0x3ffe, 0x80000000L, 0x00001000L), HEXCONSTE(-5.56268464626800345773e-309, 0xbbff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.94693962619380170427e-308, 0xbc00, 0xe0000000L, 0x00001000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.45014771701440078992e-308, 0x3c01, 0xffffffffL, 0xffffe000L), HEXCONSTE(5.00000000000000111022e-01, 0x3ffe, 0x80000000L, 0x00000800L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(-4.45014771701440078992e-308, 0xbc01, 0xffffffffL, 0xffffe000L), HEXCONSTE(5.00000000000000111022e-01, 0x3ffe, 0x80000000L, 0x00000800L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720088902e-308, 0xbc00, 0xffffffffL, 0xfffff000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(5.31250000000000000000e-01, 0x3ffe, 0x88000000L, 0x00000000L), HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(5.31250000000000000000e-01, 0x3ffe, 0x88000000L, 0x00000000L), HEXCONSTE(-2.22507385850720088902e-308, 0xbc00, 0xffffffffL, 0xfffff000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720088902e-308, 0xbc00, 0xffffffffL, 0xfffff000L), HEXCONSTE(-2.22507385850720088902e-308, 0xbc00, 0xffffffffL, 0xfffff000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720088902e-308, 0xbc00, 0xffffffffL, 0xfffff000L), HEXCONSTE(-2.22507385850720088902e-308, 0xbc00, 0xffffffffL, 0xfffff000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.99999999999999777955e-01, 0xbffe, 0xffffffffL, 0xfffff000L), HEXCONSTE(1.23259516440783094596e-32, 0x3f95, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.99999999999999777955e-01, 0x3ffe, 0xffffffffL, 0xfffff000L), HEXCONSTE(-1.23259516440783094596e-32, 0xbf95, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.99999999999999777955e-01, 0x3ffe, 0xffffffffL, 0xfffff000L), HEXCONSTE(-1.23259516440783094596e-32, 0xbf95, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.99999999999999777955e-01, 0xbffe, 0xffffffffL, 0xfffff000L), HEXCONSTE(1.23259516440783094596e-32, 0x3f95, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(2.77555756156289196736e-17, 0x3fc8, 0x80000000L, 0x00000800L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(-2.77555756156289196736e-17, 0xbfc8, 0x80000000L, 0x00000800L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), HEXCONSTE(8.98846567431157953865e+307, 0x43fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(2.77555756156289196736e-17, 0x3fc8, 0x80000000L, 0x00000800L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(-2.77555756156289196736e-17, 0xbfc8, 0x80000000L, 0x00000800L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.98846567431157953865e+307, 0xc3fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(2.77555756156289196736e-17, 0x3fc8, 0x80000000L, 0x00000800L), HEXCONSTE(9.97920154767359905828e+291, 0x43c9, 0x80000000L, 0x00000000L), HEXCONSTE(9.97920154767359905828e+291, 0x43c9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(-2.77555756156289196736e-17, 0xbfc8, 0x80000000L, 0x00000800L), HEXCONSTE(9.97920154767359905828e+291, 0x43c9, 0x80000000L, 0x00000000L), HEXCONSTE(9.97920154767359905828e+291, 0x43c9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(2.77555756156289196736e-17, 0x3fc8, 0x80000000L, 0x00000800L), HEXCONSTE(-9.97920154767359905828e+291, 0xc3c9, 0x80000000L, 0x00000000L), HEXCONSTE(-9.97920154767359905828e+291, 0xc3c9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720187716e-308, 0x3c01, 0x80000000L, 0x00000800L), HEXCONSTE(-2.77555756156289196736e-17, 0xbfc8, 0x80000000L, 0x00000800L), HEXCONSTE(-9.97920154767359905828e+291, 0xc3c9, 0x80000000L, 0x00000000L), HEXCONSTE(-9.97920154767359905828e+291, 0xc3c9, 0x80000000L, 0x00000000L), 0 },
#endif
#if defined (TEST_LDOUBLE) && LDBL_MANT_DIG == 64
	{ __LINE__, HEXCONSTE(-3.24188838352884987521e+1113, 0xce72, 0x803fc000L, 0x00000000L), HEXCONSTE(7.58484317697911551527e-1848, 0x2806, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.47152251874819186499e-737, 0x3670, 0x83ffffffL, 0xffffffffL), HEXCONSTE(-2.45644997611491690729e-734, 0xb67a, 0x801ec000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(9.84695762613665278681e+612, 0x47f3, 0x9fc00000L, 0x00000000L), HEXCONSTE(-1.91305453305165915613e-1087, 0xb1e5, 0x8000e1f0L, 0x00ff800fL), HEXCONSTE(-7.54669553718169991629e-475, 0xb9d7, 0xffffffffL, 0xffffc0ffL), HEXCONSTE(-2.63844624606300283282e-474, 0xb9d9, 0xdfc119fbL, 0x093ed092L), 0 },
	{ __LINE__, HEXCONSTE(4.64580802288587437207e-359, 0x3b58, 0xc7fc0000L, 0x03ffffffL), HEXCONSTE(6.25387407867252868367e+4614, 0x7be1, 0x81e0003fL, 0xffffffffL), HEXCONSTE(-1.11773801122906527630e+4237, 0xf6fa, 0x8fffe000L, 0x00000000L), HEXCONSTE(2.90542983688148396412e+4256, 0x773a, 0xcae9f164L, 0x020effffL), 0 },
	{ __LINE__, HEXCONSTE(-1.42897238002446878212e+542, 0xc708, 0x80001fc0L, 0x00000003L), HEXCONSTE(6.05746502655573363042e-671, 0x374c, 0xc0000000L, 0x00000000L), HEXCONSTE(1.17671232669529647052e-140, 0x3e2e, 0x8f7e0000L, 0x00000007L), HEXCONSTE(-8.65595021590056155688e-129, 0xbe55, 0xc0002f9fL, 0xfee10404L), 0 },
	{ __LINE__, HEXCONSTE(3.25864703793047743153e+3050, 0x6794, 0xc0000000L, 0x000007ffL), HEXCONSTE(-2.92395563241758324667e+1334, 0xd150, 0x80000000L, 0x00000001L), HEXCONSTE(8.48197122819917944650e+4369, 0x78b3, 0xc0700000L, 0x0001ffffL), HEXCONSTE(-9.52813936061768503789e+4384, 0xf8e5, 0xbfffffffL, 0xffffd7e4L), 0 },
	{ __LINE__, HEXCONSTE(1.14522654783539566347e-1437, 0x2d59, 0xbffff000L, 0x00000000L), HEXCONSTE(2.20181253616229718191e-3495, 0x12a6, 0x8000000fL, 0xffffffffL), HEXCONSTE(-9.19389197395040574310e-4934, 0x8000, 0x03800fffL, 0x80000000L), HEXCONSTE(2.42963525003034082019e-4932, 0x0000, 0x5c7fe80cL, 0x7ffeffffL), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(4.20262892889011688283e-4932, 0x0001, 0xa0000000L, 0x00000000L), HEXCONSTE(5.00000000000000000108e-01, 0x3ffe, 0x80000000L, 0x00000002L), HEXCONSTE(8.40525785778023376566e-4933, 0x0000, 0x20000000L, 0x00000000L), HEXCONSTE(2.94184025022308181834e-4932, 0x0000, 0x70000000L, 0x00000001L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-4.20262892889011688283e-4932, 0x8001, 0xa0000000L, 0x00000000L), HEXCONSTE(5.00000000000000000108e-01, 0x3ffe, 0x80000000L, 0x00000002L), HEXCONSTE(-8.40525785778023376566e-4933, 0x8000, 0x20000000L, 0x00000000L), HEXCONSTE(-2.94184025022308181834e-4932, 0x8000, 0x70000000L, 0x00000001L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(6.72420628622418701107e-4932, 0x0001, 0xffffffffL, 0xfffffffcL), HEXCONSTE(5.00000000000000000054e-01, 0x3ffe, 0x80000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(-6.72420628622418701107e-4932, 0x8001, 0xffffffffL, 0xfffffffcL), HEXCONSTE(5.00000000000000000054e-01, 0x3ffe, 0x80000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350590e-4932, 0x8000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(5.31250000000000000000e-01, 0x3ffe, 0x88000000L, 0x00000000L), HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(5.31250000000000000000e-01, 0x3ffe, 0x88000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350590e-4932, 0x8000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), UNDERFLOW_EXCEPTION_BEFORE_ROUNDING },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350590e-4932, 0x8000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(-3.36210314311209350590e-4932, 0x8000, 0x7fffffffL, 0xffffffffL), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350590e-4932, 0x8000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(-3.36210314311209350590e-4932, 0x8000, 0x7fffffffL, 0xffffffffL), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), UNDERFLOW_EXCEPTION },
	{ __LINE__, HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.99999999999999999892e-01, 0xbffe, 0xffffffffL, 0xfffffffeL), HEXCONSTE(2.93873587705571876992e-39, 0x3f7f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.99999999999999999892e-01, 0x3ffe, 0xffffffffL, 0xfffffffeL), HEXCONSTE(-2.93873587705571876992e-39, 0xbf7f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.99999999999999999892e-01, 0x3ffe, 0xffffffffL, 0xfffffffeL), HEXCONSTE(-2.93873587705571876992e-39, 0xbf7f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.99999999999999999892e-01, 0xbffe, 0xffffffffL, 0xfffffffeL), HEXCONSTE(2.93873587705571876992e-39, 0x3f7f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(1.35525271560688054266e-20, 0x3fbd, 0x80000000L, 0x00000001L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(-1.35525271560688054266e-20, 0xbfbd, 0x80000000L, 0x00000001L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), HEXCONSTE(5.94865747678615882543e+4931, 0x7ffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(1.35525271560688054266e-20, 0x3fbd, 0x80000000L, 0x00000001L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(-1.35525271560688054266e-20, 0xbfbd, 0x80000000L, 0x00000001L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.94865747678615882543e+4931, 0xfffe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(1.35525271560688054266e-20, 0x3fbd, 0x80000000L, 0x00000001L), HEXCONSTE(3.22477367985184627972e+4912, 0x7fbe, 0x80000000L, 0x00000000L), HEXCONSTE(3.22477367985184627972e+4912, 0x7fbe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(-1.35525271560688054266e-20, 0xbfbd, 0x80000000L, 0x00000001L), HEXCONSTE(3.22477367985184627972e+4912, 0x7fbe, 0x80000000L, 0x00000000L), HEXCONSTE(3.22477367985184627972e+4912, 0x7fbe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(1.35525271560688054266e-20, 0x3fbd, 0x80000000L, 0x00000001L), HEXCONSTE(-3.22477367985184627972e+4912, 0xffbe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.22477367985184627972e+4912, 0xffbe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350663e-4932, 0x0001, 0x80000000L, 0x00000001L), HEXCONSTE(-1.35525271560688054266e-20, 0xbfbd, 0x80000000L, 0x00000001L), HEXCONSTE(-3.22477367985184627972e+4912, 0xffbe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.22477367985184627972e+4912, 0xffbe, 0x80000000L, 0x00000000L), 0 },
#endif

};


int main(int argc, char **argv)
{
	int status;
	
	status = 0;
	
	test_init(argc, argv);
	
	status |= test_table_fff_f(fma_data, ARRAY_SIZE(fma_data), __FILE__);
	
	return status ? EXIT_FAILURE : EXIT_SUCCESS;
}

#else

int main(void)
{
	return 77;
}

#endif
