#include "testconfig.h"
#define TEST_FUNC_FF_F __MATH_PRECNAME(remainder)
#include "testdriver.h"


#define MAX_2_P HEXCONSTE(+5.94865747678615882511e+4931, 0x7ffd, 0xffffffffL, 0xffffffffL) /* LDBL_MAX / 2 */
#define MAX_4_P HEXCONSTE(+2.97432873839307941255e+4931, 0x7ffc, 0xffffffffL, 0xffffffffL) /* LDBL_MAX / 4 */
#define MAX_8_P HEXCONSTE(+1.48716436919653970628e+4931, 0x7ffb, 0xffffffffL, 0xffffffffL) /* LDBL_MAX / 8 */
#define MAX_2_M HEXCONSTE(-5.94865747678615882511e+4931, 0xfffd, 0xffffffffL, 0xffffffffL) /* -LDBL_MAX / 2 */
#define MAX_4_M HEXCONSTE(-2.97432873839307941255e+4931, 0xfffc, 0xffffffffL, 0xffffffffL) /* -LDBL_MAX / 4 */
#define MAX_8_M HEXCONSTE(-1.48716436919653970628e+4931, 0xfffb, 0xffffffffL, 0xffffffffL) /* -LDBL_MAX / 8 */


static test_ff_f_data const remainder_data[] = {
	/* remainder (x, +0) == qNaN plus invalid exception.  */
	{ __LINE__, HEXCONSTE(3, 0x4000, 0xc0000000L, 0x00000000L), ZERO_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(-1.1, 0xbfff, 0x8cccccccL, 0xcccccccdL), ZERO_P, QNAN_P, 0 },
	{ __LINE__, ZERO_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, ZERO_M, ZERO_P, QNAN_P, 0 },
	{ __LINE__, SUBNORM_P, ZERO_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, MIN_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, MAX_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, INF_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, INF_M, ZERO_P, QNAN_P, 0 },

	/* remainder (x, -0) == qNaN plus invalid exception.  */
	{ __LINE__, HEXCONSTE(3, 0x4000, 0xc0000000L, 0x00000000L), ZERO_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(-1.1, 0xbfff, 0x8cccccccL, 0xcccccccdL), ZERO_M, QNAN_P, 0 },
	{ __LINE__, ZERO_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, ZERO_M, ZERO_M, QNAN_P, 0 },
	{ __LINE__, SUBNORM_P, ZERO_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, MIN_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, MAX_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, INF_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, INF_M, ZERO_M, QNAN_P, 0 },
	
	/* remainder (+inf, y) == qNaN plus invalid exception.  */
	{ __LINE__, INF_P, HEXCONSTE(3, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, INF_P, HEXCONSTE(-1.1, 0xbfff, 0x8cccccccL, 0xcccccccdL), QNAN_P, 0 },
	{ __LINE__, INF_P, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, INF_P, MIN_P, QNAN_P, 0 },
	{ __LINE__, INF_P, MAX_P, QNAN_P, 0 },
	{ __LINE__, INF_P, INF_P, QNAN_P, 0 },
	{ __LINE__, INF_P, INF_M, QNAN_P, 0 },

	/* remainder (-inf, y) == qNaN plus invalid exception.  */
	{ __LINE__, INF_M, HEXCONSTE(3, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, INF_M, HEXCONSTE(-1.1, 0xbfff, 0x8cccccccL, 0xcccccccdL), QNAN_P, 0 },
	{ __LINE__, INF_M, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, INF_M, MIN_P, QNAN_P, 0 },
	{ __LINE__, INF_M, MAX_P, QNAN_P, 0 },
	{ __LINE__, INF_M, INF_P, QNAN_P, 0 },
	{ __LINE__, INF_M, INF_M, QNAN_P, 0 },

	/* remainder (+0, y) == +0 for y != 0.  */
	{ __LINE__, ZERO_P, HEXCONSTE(3, 0x4000, 0xc0000000L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, ZERO_P, SUBNORM_P, ZERO_P, FLAG_SUBNORM },
	{ __LINE__, ZERO_P, SUBNORM_M, ZERO_P, FLAG_SUBNORM },
	{ __LINE__, ZERO_P, MIN_P, ZERO_P, FLAG_MINVAL },
	{ __LINE__, ZERO_P, MIN_M, ZERO_P, FLAG_MINVAL },
	{ __LINE__, ZERO_P, MAX_P, ZERO_P, 0 },
	{ __LINE__, ZERO_P, MAX_M, ZERO_P, 0 },
	
	/* remainder (-0, y) == -0 for y != 0.  */
	{ __LINE__, ZERO_M, HEXCONSTE(3, 0x4000, 0xc0000000L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM|FLAG_FAIL_STONX },
	{ __LINE__, ZERO_M, SUBNORM_P, ZERO_M, FLAG_SUBNORM },
	{ __LINE__, ZERO_M, SUBNORM_M, ZERO_M, FLAG_SUBNORM },
	{ __LINE__, ZERO_M, MIN_P, ZERO_M, FLAG_MINVAL },
	{ __LINE__, ZERO_M, MIN_M, ZERO_M, FLAG_MINVAL },
	{ __LINE__, ZERO_M, MAX_P, ZERO_M, FLAG_FAIL_STONX },
	{ __LINE__, ZERO_M, MAX_M, ZERO_M, FLAG_FAIL_STONX },
	
	/* remainder (x, +inf) == x for x not infinite.  */
	{ __LINE__, ZERO_P, INF_P, ZERO_P, FLAG_FAIL_ARANYM|FLAG_FAIL_STONX|FLAG_XFAIL_LINUX },
	{ __LINE__, ZERO_M, INF_P, ZERO_M, FLAG_FAIL_ARANYM|FLAG_FAIL_STONX|FLAG_XFAIL_LINUX },
	{ __LINE__, SUBNORM_P, INF_P, SUBNORM_P, FLAG_SUBNORM },
	{ __LINE__, MIN_P, INF_P, MIN_P, FLAG_MINVAL },
	{ __LINE__, MAX_P, INF_P, MAX_P, FLAG_MINVAL },
	{ __LINE__, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), INF_P, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), FLAG_FAIL_ARANYM|FLAG_FAIL_STONX|FLAG_XFAIL_LINUX },

	/* remainder (x, -inf) == x for x not infinite.  */
	{ __LINE__, ZERO_P, INF_M, ZERO_P, FLAG_FAIL_ARANYM|FLAG_FAIL_STONX|FLAG_XFAIL_LINUX },
	{ __LINE__, ZERO_M, INF_M, ZERO_M, FLAG_FAIL_ARANYM|FLAG_FAIL_STONX|FLAG_XFAIL_LINUX },
	{ __LINE__, SUBNORM_P, INF_M, SUBNORM_P, FLAG_SUBNORM },
	{ __LINE__, MIN_P, INF_M, MIN_P, FLAG_MINVAL },
	{ __LINE__, MAX_P, INF_M, MAX_P, FLAG_MINVAL },
	{ __LINE__, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), INF_M, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), FLAG_FAIL_ARANYM|FLAG_FAIL_STONX|FLAG_XFAIL_LINUX },
	
	{ __LINE__, ZERO_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), SNAN_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), SNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, ZERO_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, ZERO_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, QNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, QNAN_P, INF_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, INF_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, INF_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, INF_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, ZERO_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, ZERO_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, SNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, SNAN_P, INF_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, INF_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, INF_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, INF_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, SNAN_M, QNAN_P, 0 },
	
	{ __LINE__, MAX_P, MAX_P, ZERO_P, 0 },
	{ __LINE__, MAX_P, MAX_M, ZERO_P, 0 },
	{ __LINE__, MAX_P, MIN_P, ZERO_P, FLAG_MINVAL },
	{ __LINE__, MAX_P, MIN_M, ZERO_P, FLAG_MINVAL },
	{ __LINE__, MAX_P, SUBNORM_P, ZERO_P, FLAG_SUBNORM },
	{ __LINE__, MAX_P, SUBNORM_M, ZERO_P, FLAG_SUBNORM },
	{ __LINE__, MAX_M, MAX_P, ZERO_M, FLAG_FAIL_STONX },
	{ __LINE__, MAX_M, MAX_M, ZERO_M, FLAG_FAIL_STONX },
	{ __LINE__, MAX_M, MIN_P, ZERO_M, FLAG_MINVAL },
	{ __LINE__, MAX_M, MIN_M, ZERO_M, FLAG_MINVAL },
	{ __LINE__, MAX_M, SUBNORM_P, ZERO_M, FLAG_SUBNORM },
	{ __LINE__, MAX_M, SUBNORM_M, ZERO_M, FLAG_SUBNORM },
	
	{ __LINE__, MIN_P, MAX_P, MIN_P, FLAG_MINVAL },
	{ __LINE__, MIN_P, MAX_M, MIN_P, FLAG_MINVAL },
	{ __LINE__, MIN_P, MIN_P, ZERO_P, FLAG_MINVAL },
	{ __LINE__, MIN_P, MIN_M, ZERO_P, FLAG_MINVAL },
	{ __LINE__, MIN_P, SUBNORM_P, ZERO_P, FLAG_MINVAL },
	{ __LINE__, MIN_P, SUBNORM_M, ZERO_P, FLAG_MINVAL },
	{ __LINE__, MIN_M, MAX_P, MIN_M, FLAG_MINVAL },
	{ __LINE__, MIN_M, MAX_M, MIN_M, FLAG_MINVAL },
	{ __LINE__, MIN_M, MIN_P, ZERO_M, FLAG_MINVAL },
	{ __LINE__, MIN_M, MIN_M, ZERO_M, FLAG_MINVAL },
	{ __LINE__, MIN_M, SUBNORM_P, ZERO_M, FLAG_MINVAL },
	{ __LINE__, MIN_M, SUBNORM_M, ZERO_M, FLAG_MINVAL },
	
	{ __LINE__, SUBNORM_P, MAX_P, SUBNORM_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, MAX_M, SUBNORM_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, MIN_P, SUBNORM_P, FLAG_MINVAL },
	{ __LINE__, SUBNORM_P, MIN_M, SUBNORM_P, FLAG_MINVAL },
	{ __LINE__, SUBNORM_P, SUBNORM_P, ZERO_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, SUBNORM_M, ZERO_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, MAX_P, SUBNORM_M, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, MAX_M, SUBNORM_M, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, MIN_P, SUBNORM_M, FLAG_MINVAL },
	{ __LINE__, SUBNORM_M, MIN_M, SUBNORM_M, FLAG_MINVAL },
	{ __LINE__, SUBNORM_M, SUBNORM_P, ZERO_M, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, SUBNORM_M, ZERO_M, FLAG_SUBNORM },

	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_2_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_2_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_4_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_4_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_8_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), MAX_8_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_2_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_2_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_4_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_4_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_8_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), MAX_8_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, MAX_P, MAX_2_P, ZERO_P, 0 },
	{ __LINE__, MAX_P, MAX_2_M, ZERO_P, 0 },
	{ __LINE__, MAX_M, MAX_2_P, ZERO_M, FLAG_FAIL_STONX|FLAG_FAIL_ARANYM }, /* should be ZERO_P */
	{ __LINE__, MAX_M, MAX_2_M, ZERO_M, FLAG_FAIL_STONX|FLAG_FAIL_ARANYM }, /* should be ZERO_P */
	
	{ __LINE__, HEXCONSTE(2, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(2, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(-2, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), ZERO_M, FLAG_FAIL_STONX|FLAG_FAIL_ARANYM }, /* should be ZERO_P */
	{ __LINE__, HEXCONSTE(-2, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), ZERO_M, FLAG_FAIL_STONX|FLAG_FAIL_ARANYM }, /* should be ZERO_P */
	
	{ __LINE__, HEXCONSTE(6.5, 0x4001, 0xd0000000L, 0x00000000L), HEXCONSTE(2.25L, 0x4000, 0x90000000L, 0x00000000L), HEXCONSTE(-2.50000000000000000000e-01, 0xbffd, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-6.5, 0xc001, 0xd0000000L, 0x00000000L), HEXCONSTE(2.25L, 0x4000, 0x90000000L, 0x00000000L), HEXCONSTE(2.50000000000000000000e-01, 0x3ffd, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(6.5, 0x4001, 0xd0000000L, 0x00000000L), HEXCONSTE(-2.25L, 0xc000, 0x90000000L, 0x00000000L), HEXCONSTE(-2.50000000000000000000e-01, 0xbffd, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-6.5, 0xc001, 0xd0000000L, 0x00000000L), HEXCONSTE(-2.25L, 0xc000, 0x90000000L, 0x00000000L), HEXCONSTE(2.50000000000000000000e-01, 0x3ffd, 0x80000000L, 0x00000000L), 0 },
	
	{ __LINE__, HEXCONSTE(1.17549421069244107549e-38, 0x3f80, 0xfffffe00L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720088902e-308, 0x3c00, 0xffffffffL, 0xfffff000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), ZERO_P, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(3.36210314311209350590e-4932, 0x0000, 0x7fffffffL, 0xffffffffL), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), ZERO_P, 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.00000000000000005551e+00, 0xbfff, 0x80000000L, 0x00000200L), HEXCONSTE(9.99999999999999944489e-01, 0x3ffe, 0xffffffffL, 0xfffffc00L), HEXCONSTE(-1.11022302462515654042e-16, 0xbfca, 0x80000000L, 0x00000000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(9.99999999999999958367e-01, 0x3ffe, 0xffffffffL, 0xfffffd00L), HEXCONSTE(9.99999999999999944489e-01, 0x3ffe, 0xffffffffL, 0xfffffc00L), HEXCONSTE(1.38777878078144567553e-17, 0x3fc7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99999999999999958367e-01, 0xbffe, 0xffffffffL, 0xfffffd00L), HEXCONSTE(9.99999999999999944489e-01, 0x3ffe, 0xffffffffL, 0xfffffc00L), HEXCONSTE(-1.38777878078144567553e-17, 0xbfc7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(9.99999999999999958367e-01, 0x3ffe, 0xffffffffL, 0xfffffd00L), HEXCONSTE(-9.99999999999999944489e-01, 0xbffe, 0xffffffffL, 0xfffffc00L), HEXCONSTE(1.38777878078144567553e-17, 0x3fc7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-9.99999999999999958367e-01, 0xbffe, 0xffffffffL, 0xfffffd00L), HEXCONSTE(-9.99999999999999944489e-01, 0xbffe, 0xffffffffL, 0xfffffc00L), HEXCONSTE(-1.38777878078144567553e-17, 0xbfc7, 0x80000000L, 0x00000000L), 0 },
#endif

	{ __LINE__, HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.0, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.0, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },

	{ __LINE__, HEXCONSTE(1.625, 0x3fff, 0xd0000000L, 0x00000000L), HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-0.375, 0xbffd, 0xc0000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.625, 0xbfff, 0xd0000000L, 0x00000000L), HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(0.375, 0x3ffd, 0xc0000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.625, 0x3fff, 0xd0000000L, 0x00000000L), HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-0.375, 0xbffd, 0xc0000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.625, 0xbfff, 0xd0000000L, 0x00000000L), HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(0.375, 0x3ffd, 0xc0000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(5.0, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_STONX|FLAG_FAIL_ARANYM }, /* returns -1, which is OK */
	{ __LINE__, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(-3.00000000000000005551e+00, 0xc000, 0xc0000000L, 0x00000100L), HEXCONSTE(2.0, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999944489e-01, 0x3ffe, 0xffffffffL, 0xfffffc00L), 0 },
	{ __LINE__, HEXCONSTE(9.99999999999999986122e-01, 0x3ffe, 0xffffffffL, 0xffffff00L), HEXCONSTE(-1.00000000000000001388e+00, 0xbfff, 0x80000000L, 0x00000080L), HEXCONSTE(-2.77555756156289135106e-17, 0xbfc8, 0x80000000L, 0x00000000L), 0 },
#endif

	{ __LINE__, HEXCONSTE(3419, 0x400a, 0xd5b00000L, 0x00000000L), HEXCONSTE(360, 0x4007, 0xb4000000L, 0x00000000L), HEXCONSTE(179, 0x4006, 0xb3000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3419, 0xc00a, 0xd5b00000L, 0x00000000L), HEXCONSTE(360, 0x4007, 0xb4000000L, 0x00000000L), HEXCONSTE(-179, 0xc006, 0xb3000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3419, 0x400a, 0xd5b00000L, 0x00000000L), HEXCONSTE(-360, 0xc007, 0xb4000000L, 0x00000000L), HEXCONSTE(179, 0x4006, 0xb3000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3419, 0xc00a, 0xd5b00000L, 0x00000000L), HEXCONSTE(-360, 0xc007, 0xb4000000L, 0x00000000L), HEXCONSTE(-179, 0xc006, 0xb3000000L, 0x00000000L), 0 },
};


int main(int argc, char **argv)
{
	int status;
	
	status = 0;
	
	test_init(argc, argv);
	
	status |= test_table_ff_f(remainder_data, ARRAY_SIZE(remainder_data), __FILE__);
	
	return status ? EXIT_FAILURE : EXIT_SUCCESS;
}
