#include "testconfig.h"
#define TEST_FUNC_FF_F __MATH_PRECNAME(atan2)
#include "testdriver.h"



static test_ff_f_data const fatan2_data[] = {
	/* atan2 (0,x) == 0 for x > 0.	*/
	{ __LINE__, ZERO_P, HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, ZERO_P, ZERO_P, ZERO_P, FLAG_FAIL_HARDFLOAT }, /* not handled by inline version in math-68881.h */
	/* atan2 (-0,x) == -0 for x > 0.  */
	{ __LINE__, ZERO_M, HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM2 },
	{ __LINE__, ZERO_M, ZERO_P, ZERO_M, FLAG_FAIL_HARDFLOAT }, /* not handled by inline version in math-68881.h */
	/* atan2 (+0,x) == +pi for x < 0.  */
	{ __LINE__, ZERO_P, HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, ZERO_P, ZERO_M, HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_FAIL_HARDFLOAT }, /* not handled by inline version in math-68881.h */
	/* atan2 (-0,x) == -pi for x < 0.  */
	{ __LINE__, ZERO_M, HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_FAIL_HARDFLOAT }, /* not handled by inline version in math-68881.h */
	{ __LINE__, ZERO_M, ZERO_M, HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_FAIL_HARDFLOAT }, /* not handled by inline version in math-68881.h */

	/* atan2 (y,inf) == +0 for finite y > 0 or +0. */
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), INF_P, ZERO_P, FLAG_XFAIL_LINUX },
	{ __LINE__, ZERO_P, INF_P, ZERO_P, FLAG_XFAIL_LINUX },
	{ __LINE__, MIN_P, INF_P, ZERO_P, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, SUBNORM_P, INF_P, ZERO_P, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
	{ __LINE__, MAX_P, INF_P, ZERO_P, FLAG_XFAIL_LINUX },
	
	/* atan2 (y,inf) == -0 for finite y < 0 or -0.  */
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), INF_P, ZERO_M, FLAG_XFAIL_LINUX|FLAG_FAIL_ARANYM2 },
	{ __LINE__, ZERO_M, INF_P, ZERO_M, FLAG_XFAIL_LINUX|FLAG_FAIL_ARANYM2 },
	{ __LINE__, MIN_M, INF_P, ZERO_M, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, SUBNORM_M, INF_P, ZERO_M, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
	{ __LINE__, MAX_M, INF_P, ZERO_M, FLAG_XFAIL_LINUX },

 	/* atan2(+inf, x) == pi/2 for finite x.  */
	{ __LINE__, INF_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), HEXCONST_PI_2, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, ZERO_P, HEXCONST_PI_2, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, MIN_P, HEXCONST_PI_2, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, SUBNORM_P, HEXCONST_PI_2, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, MAX_P, HEXCONST_PI_2, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), HEXCONST_PI_2, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, ZERO_M, HEXCONST_PI_2, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, SUBNORM_M, HEXCONST_PI_2, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, INF_P, MIN_M, HEXCONST_PI_2, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, INF_P, MAX_M, HEXCONST_PI_2, FLAG_XFAIL_LINUX },
#endif

	/* atan2(-inf, x) == -pi/2 for finite x.  */
	{ __LINE__, INF_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), HEXCONST_PI_2_M, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, ZERO_P, HEXCONST_PI_2_M, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, MIN_P, HEXCONST_PI_2_M, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, SUBNORM_P, HEXCONST_PI_2_M, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, MAX_P, HEXCONST_PI_2_M, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), HEXCONST_PI_2_M, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, ZERO_M, HEXCONST_PI_2_M, FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, MIN_M, HEXCONST_PI_2_M, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, INF_M, SUBNORM_M, HEXCONST_PI_2_M, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, INF_M, MAX_M, HEXCONST_PI_2_M, FLAG_XFAIL_LINUX },
#endif

	/* atan2 (y,-inf) == +pi for finite y > 0 or +0.  */
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), INF_M, HEXCONST_PI, FLAG_XFAIL_LINUX },
	{ __LINE__, ZERO_P, INF_M, HEXCONST_PI, FLAG_XFAIL_LINUX },
	{ __LINE__, MIN_P, INF_M, HEXCONST_PI, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, SUBNORM_P, INF_M, HEXCONST_PI, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
	{ __LINE__, MAX_P, INF_M, HEXCONST_PI, FLAG_XFAIL_LINUX },

	/* atan2 (y,-inf) == -pi for finite y < 0 or -0.  */
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), INF_M, HEXCONST_PI_M, FLAG_XFAIL_LINUX },
	{ __LINE__, ZERO_M, INF_M, HEXCONST_PI_M, FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* not handled by inline version in math-68881.h */
	{ __LINE__, MIN_M, INF_M, HEXCONST_PI_M, FLAG_MINVAL|FLAG_XFAIL_LINUX },
	{ __LINE__, SUBNORM_M, INF_M, HEXCONST_PI_M, FLAG_SUBNORM|FLAG_XFAIL_LINUX },
	{ __LINE__, MAX_M, INF_M, HEXCONST_PI_M, FLAG_XFAIL_LINUX },

	/* If y is positive infinity (negative infinity) and x is positive infinity, +pi/4 (-pi/4) is returned. */
	{ __LINE__, INF_P, INF_P, HEXCONST_PI_4, FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* not handled by inline version in math-68881.h */
	{ __LINE__, INF_M, INF_P, HEXCONST_PI_4_M, FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* not handled by inline version in math-68881.h */
	/* If y is positive infinity (negative infinity) and x is negative infinity, +3*pi/4 (-3*pi/4) is returned. */
	{ __LINE__, INF_P, INF_M, HEXCONSTE(2.356194490192344928846982537459627163, 0x4000, 0x96cbe3f9L, 0x990e91a8L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* not handled by inline version in math-68881.h */
	{ __LINE__, INF_M, INF_M, HEXCONSTE(-2.356194490192344928846982537459627163, 0xc000, 0x96cbe3f9L, 0x990e91a8L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* not handled by inline version in math-68881.h */

	{ __LINE__, QNAN_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, INF_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, INF_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, QNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, QNAN_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, QNAN_P, MIN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, QNAN_P, MIN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, QNAN_P, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, QNAN_P, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, QNAN_P, MAX_P, QNAN_P, 0 },
	{ __LINE__, QNAN_P, MAX_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, INF_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, INF_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, ZERO_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, ZERO_M, QNAN_P, 0 },
	{ __LINE__, QNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, QNAN_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, QNAN_M, MIN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, QNAN_M, MIN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, QNAN_M, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, QNAN_M, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, QNAN_M, MAX_P, QNAN_P, 0 },
	{ __LINE__, QNAN_M, MAX_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, INF_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, INF_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, ZERO_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, ZERO_M, QNAN_P, 0 },
	{ __LINE__, SNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, SNAN_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, SNAN_P, MIN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SNAN_P, MIN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SNAN_P, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SNAN_P, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SNAN_P, MAX_P, QNAN_P, 0 },
	{ __LINE__, SNAN_P, MAX_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, INF_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, INF_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, ZERO_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, ZERO_M, QNAN_P, 0 },
	{ __LINE__, SNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, SNAN_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__, SNAN_M, MIN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SNAN_M, MIN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SNAN_M, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SNAN_M, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SNAN_M, MAX_P, QNAN_P, 0 },
	{ __LINE__, SNAN_M, MAX_M, QNAN_P, 0 },
	{ __LINE__, INF_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, QNAN_P, 0 },
	{ __LINE__, MIN_P, QNAN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, MIN_M, QNAN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SUBNORM_P, QNAN_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, QNAN_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, MAX_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__, MAX_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_M, QNAN_P, 0 },
	{ __LINE__, MIN_P, QNAN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, MIN_M, QNAN_M, QNAN_P, FLAG_MINVAL},
	{ __LINE__, SUBNORM_P, QNAN_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, QNAN_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, MAX_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__, MAX_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, ZERO_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), SNAN_P, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), SNAN_P, QNAN_P, 0 },
	{ __LINE__, MIN_P, SNAN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, MIN_M, SNAN_P, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SUBNORM_P, SNAN_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, SNAN_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, MAX_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__, MAX_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__, INF_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, INF_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, ZERO_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), SNAN_M, QNAN_P, 0 },
	{ __LINE__, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), SNAN_M, QNAN_P, 0 },
	{ __LINE__, MIN_P, SNAN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, MIN_M, SNAN_M, QNAN_P, FLAG_MINVAL },
	{ __LINE__, SUBNORM_P, SNAN_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, SNAN_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__, MAX_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__, MAX_M, SNAN_M, QNAN_P, 0 },

	/* atan2 (y,+0) == pi/2 for y > 0.	*/
	{ __LINE__, HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), ZERO_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	/* atan2 (y,-0) == pi/2 for y > 0.	*/
	{ __LINE__, HEXCONSTE(1.0, 0x3fff, 0x80000000L, 0x00000000L), ZERO_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	/* atan2 (y,+0) == -pi/2 for y < 0.  */
	{ __LINE__, HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), ZERO_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	/* atan2 (y,-0) == -pi/2 for y < 0.  */
	{ __LINE__, HEXCONSTE(-1.0, 0xbfff, 0x80000000L, 0x00000000L), ZERO_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },

	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.89288338504219125616e-270, 0x3c7e, 0xffffff00L, 0x00000800L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.86016086794739201471e-4894, 0x007e, 0xffffff00L, 0x00000001L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.51100743476790606040e-4624, 0x03fe, 0xffffffffL, 0xfffff801L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.89288338504219125616e-270, 0xbc7e, 0xffffff00L, 0x00000800L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.86016086794739201471e-4894, 0x807e, 0xffffff00L, 0x00000001L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.51100743476790606040e-4624, 0x83fe, 0xffffffffL, 0xfffff801L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(6.43501108793284386797e-01, 0x3ffe, 0xa4bc7d19L, 0x34f70924L), 0 },
	{ __LINE__, HEXCONSTE(-7.50000000000000000000e-01, 0xbffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-6.43501108793284386797e-01, 0xbffe, 0xa4bc7d19L, 0x34f70924L), 0 },
	{ __LINE__, HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.49809154479650885172e+00, 0x4000, 0x9fe0bb5bL, 0xd42affecL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.50000000000000000000e-01, 0xbffe, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.49809154479650885172e+00, 0xc000, 0x9fe0bb5bL, 0xd42affecL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.90625000000000000000e-01, 0x3ffd, 0xc8000000L, 0x00000000L), HEXCONSTE(2.90000025415793061256e-04, 0x3ff3, 0x980b2500L, 0x00000000L), HEXCONSTE(1.57005392686622535347e+00, 0x3fff, 0xc8f786eeL, 0x6c6463d8L), 0 },
	{ __LINE__, HEXCONSTE(3.90625000000000000000e-01, 0x3ffd, 0xc8000000L, 0x00000000L), HEXCONSTE(2.89999996311962604523e-04, 0x3ff3, 0x980b2400L, 0x00000000L), HEXCONSTE(1.57005392694073111831e+00, 0x3fff, 0xc8f786eeL, 0x955a24ecL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.90625000000000000000e-01, 0x3ffd, 0xc8000000L, 0x00000000L), HEXCONSTE(2.90000000000000001700e-04, 0x3ff3, 0x980b2420L, 0x70b8d000L), HEXCONSTE(1.57005392693128974777e+00, 0x3fff, 0xc8f786eeL, 0x902963b3L), 0 },
	{ __LINE__, HEXCONSTE(3.90625000000000000000e-01, 0x3ffd, 0xc8000000L, 0x00000000L), HEXCONSTE(2.89999999999999947490e-04, 0x3ff3, 0x980b2420L, 0x70b8c800L), HEXCONSTE(1.57005392693128974799e+00, 0x3fff, 0xc8f786eeL, 0x902963b5L), 0 },
	{ __LINE__, HEXCONSTE(3.90625000000000000000e-01, 0x3ffd, 0xc8000000L, 0x00000000L), HEXCONSTE(2.90000000000000000006e-04, 0x3ff3, 0x980b2420L, 0x70b8cfc0L), HEXCONSTE(1.57005392693128974777e+00, 0x3fff, 0xc8f786eeL, 0x902963b3L), 0 },
	{ __LINE__, HEXCONSTE(3.90625000000000000000e-01, 0x3ffd, 0xc8000000L, 0x00000000L), HEXCONSTE(2.89999999999999999979e-04, 0x3ff3, 0x980b2420L, 0x70b8cfbfL), HEXCONSTE(1.57005392693128974777e+00, 0x3fff, 0xc8f786eeL, 0x902963b3L), 0 },
	{ __LINE__, HEXCONSTE(1.39062500000000000000e+00, 0x3fff, 0xb2000000L, 0x00000000L), HEXCONSTE(9.29687500000000000000e-01, 0x3ffe, 0xee000000L, 0x00000000L), HEXCONSTE(9.81498387184244311507e-01, 0x3ffe, 0xfb437a72L, 0x087797cfL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56826996803283691406e-03, 0xbff7, 0xf7ff4000L, 0x00000000L), HEXCONSTE(-1.79273576941341161728e-03, 0xbff5, 0xeafa3b00L, 0x00000000L), HEXCONSTE(-1.80338464370443418792e+00, 0xbfff, 0xe6d54ed9L, 0x68da2408L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56826996803283691406e-03, 0xbff7, 0xf7ff4000L, 0x00000000L), HEXCONSTE(-1.79273588582873344421e-03, 0xbff5, 0xeafa3c00L, 0x00000000L), HEXCONSTE(-1.80338465826923068200e+00, 0xbfff, 0xe6d54ef8L, 0xafef048aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56826996803283691406e-03, 0xbff7, 0xf7ff4000L, 0x00000000L), HEXCONSTE(-1.79273585753872790639e-03, 0xbff5, 0xeafa3bc1L, 0xca248800L), HEXCONSTE(-1.80338465472984973168e+00, 0xbfff, 0xe6d54ef1L, 0x16236eb5L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56826996803283691406e-03, 0xbff7, 0xf7ff4000L, 0x00000000L), HEXCONSTE(-1.79273585753872812323e-03, 0xbff5, 0xeafa3bc1L, 0xca249000L), HEXCONSTE(-1.80338465472984975878e+00, 0xbfff, 0xe6d54ef1L, 0x16236fafL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56826996803283691406e-03, 0xbff7, 0xf7ff4000L, 0x00000000L), HEXCONSTE(-1.79273585753872803599e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc8L), HEXCONSTE(-1.80338465472984974794e+00, 0xbfff, 0xe6d54ef1L, 0x16236f4bL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56826996803283691406e-03, 0xbff7, 0xf7ff4000L, 0x00000000L), HEXCONSTE(-1.79273585753872803610e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc9L), HEXCONSTE(-1.80338465472984974794e+00, 0xbfff, 0xe6d54ef1L, 0x16236f4bL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827043369412422180e-03, 0xbff7, 0xf7ff4100L, 0x00000000L), HEXCONSTE(-1.79273576941341161728e-03, 0xbff5, 0xeafa3b00L, 0x00000000L), HEXCONSTE(-1.80338462990427625041e+00, 0xbfff, 0xe6d54ebbL, 0xc62292dfL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827043369412422180e-03, 0xbff7, 0xf7ff4100L, 0x00000000L), HEXCONSTE(-1.79273588582873344421e-03, 0xbff5, 0xeafa3c00L, 0x00000000L), HEXCONSTE(-1.80338464446907194359e+00, 0xbfff, 0xe6d54edbL, 0x0d375686L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827043369412422180e-03, 0xbff7, 0xf7ff4100L, 0x00000000L), HEXCONSTE(-1.79273585753872790639e-03, 0xbff5, 0xeafa3bc1L, 0xca248800L), HEXCONSTE(-1.80338464092969118788e+00, 0xbfff, 0xe6d54ed3L, 0x736bc7b4L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827043369412422180e-03, 0xbff7, 0xf7ff4100L, 0x00000000L), HEXCONSTE(-1.79273585753872812323e-03, 0xbff5, 0xeafa3bc1L, 0xca249000L), HEXCONSTE(-1.80338464092969121499e+00, 0xbfff, 0xe6d54ed3L, 0x736bc8aeL), 0 },
	{ __LINE__, HEXCONSTE(-7.56827043369412422180e-03, 0xbff7, 0xf7ff4100L, 0x00000000L), HEXCONSTE(-1.79273585753872803599e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc8L), HEXCONSTE(-1.80338464092969120414e+00, 0xbfff, 0xe6d54ed3L, 0x736bc84aL), 0 },
	{ __LINE__, HEXCONSTE(-7.56827043369412422180e-03, 0xbff7, 0xf7ff4100L, 0x00000000L), HEXCONSTE(-1.79273585753872803610e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc9L), HEXCONSTE(-1.80338464092969120414e+00, 0xbfff, 0xe6d54ed3L, 0x736bc84aL), 0 },
	{ __LINE__, HEXCONSTE(-7.56827042671106252347e-03, 0xbff7, 0xf7ff40fcL, 0x2938b000L), HEXCONSTE(-1.79273576941341161728e-03, 0xbff5, 0xeafa3b00L, 0x00000000L), HEXCONSTE(-1.80338463011122355616e+00, 0xbfff, 0xe6d54ebcL, 0x37e7d158L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106252347e-03, 0xbff7, 0xf7ff40fcL, 0x2938b000L), HEXCONSTE(-1.79273588582873344421e-03, 0xbff5, 0xeafa3c00L, 0x00000000L), HEXCONSTE(-1.80338464467601926138e+00, 0xbfff, 0xe6d54edbL, 0x7efc956eL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106252347e-03, 0xbff7, 0xf7ff40fcL, 0x2938b000L), HEXCONSTE(-1.79273585753872790639e-03, 0xbff5, 0xeafa3bc1L, 0xca248800L), HEXCONSTE(-1.80338464113663850274e+00, 0xbfff, 0xe6d54ed3L, 0xe5310681L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106252347e-03, 0xbff7, 0xf7ff40fcL, 0x2938b000L), HEXCONSTE(-1.79273585753872812323e-03, 0xbff5, 0xeafa3bc1L, 0xca249000L), HEXCONSTE(-1.80338464113663852985e+00, 0xbfff, 0xe6d54ed3L, 0xe531077bL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106252347e-03, 0xbff7, 0xf7ff40fcL, 0x2938b000L), HEXCONSTE(-1.79273585753872803599e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc8L), HEXCONSTE(-1.80338464113663851890e+00, 0xbfff, 0xe6d54ed3L, 0xe5310716L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106252347e-03, 0xbff7, 0xf7ff40fcL, 0x2938b000L), HEXCONSTE(-1.79273585753872803610e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc9L), HEXCONSTE(-1.80338464113663851901e+00, 0xbfff, 0xe6d54ed3L, 0xe5310717L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339083e-03, 0xbff7, 0xf7ff40fcL, 0x2938b800L), HEXCONSTE(-1.79273576941341161728e-03, 0xbff5, 0xeafa3b00L, 0x00000000L), HEXCONSTE(-1.80338463011122353047e+00, 0xbfff, 0xe6d54ebcL, 0x37e7d06bL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339083e-03, 0xbff7, 0xf7ff40fcL, 0x2938b800L), HEXCONSTE(-1.79273588582873344421e-03, 0xbff5, 0xeafa3c00L, 0x00000000L), HEXCONSTE(-1.80338464467601923568e+00, 0xbfff, 0xe6d54edbL, 0x7efc9481L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339083e-03, 0xbff7, 0xf7ff40fcL, 0x2938b800L), HEXCONSTE(-1.79273585753872790639e-03, 0xbff5, 0xeafa3bc1L, 0xca248800L), HEXCONSTE(-1.80338464113663847705e+00, 0xbfff, 0xe6d54ed3L, 0xe5310594L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339083e-03, 0xbff7, 0xf7ff40fcL, 0x2938b800L), HEXCONSTE(-1.79273585753872812323e-03, 0xbff5, 0xeafa3bc1L, 0xca249000L), HEXCONSTE(-1.80338464113663850415e+00, 0xbfff, 0xe6d54ed3L, 0xe531068eL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339083e-03, 0xbff7, 0xf7ff40fcL, 0x2938b800L), HEXCONSTE(-1.79273585753872803599e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc8L), HEXCONSTE(-1.80338464113663849320e+00, 0xbfff, 0xe6d54ed3L, 0xe5310629L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339083e-03, 0xbff7, 0xf7ff40fcL, 0x2938b800L), HEXCONSTE(-1.79273585753872803610e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc9L), HEXCONSTE(-1.80338464113663849331e+00, 0xbfff, 0xe6d54ed3L, 0xe531062aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106338999e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7feL), HEXCONSTE(-1.79273576941341161728e-03, 0xbff5, 0xeafa3b00L, 0x00000000L), HEXCONSTE(-1.80338463011122353047e+00, 0xbfff, 0xe6d54ebcL, 0x37e7d06bL), FLAG_INEXACT},
	{ __LINE__, HEXCONSTE(-7.56827042671106338999e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7feL), HEXCONSTE(-1.79273588582873344421e-03, 0xbff5, 0xeafa3c00L, 0x00000000L), HEXCONSTE(-1.80338464467601923568e+00, 0xbfff, 0xe6d54edbL, 0x7efc9481L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106338999e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7feL), HEXCONSTE(-1.79273585753872790639e-03, 0xbff5, 0xeafa3bc1L, 0xca248800L), HEXCONSTE(-1.80338464113663847705e+00, 0xbfff, 0xe6d54ed3L, 0xe5310594L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106338999e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7feL), HEXCONSTE(-1.79273585753872812323e-03, 0xbff5, 0xeafa3bc1L, 0xca249000L), HEXCONSTE(-1.80338464113663850415e+00, 0xbfff, 0xe6d54ed3L, 0xe531068eL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106338999e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7feL), HEXCONSTE(-1.79273585753872803599e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc8L), HEXCONSTE(-1.80338464113663849331e+00, 0xbfff, 0xe6d54ed3L, 0xe531062aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106338999e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7feL), HEXCONSTE(-1.79273585753872803610e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc9L), HEXCONSTE(-1.80338464113663849331e+00, 0xbfff, 0xe6d54ed3L, 0xe531062aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339041e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7ffL), HEXCONSTE(-1.79273576941341161728e-03, 0xbff5, 0xeafa3b00L, 0x00000000L), HEXCONSTE(-1.80338463011122353047e+00, 0xbfff, 0xe6d54ebcL, 0x37e7d06bL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339041e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7ffL), HEXCONSTE(-1.79273588582873344421e-03, 0xbff5, 0xeafa3c00L, 0x00000000L), HEXCONSTE(-1.80338464467601923568e+00, 0xbfff, 0xe6d54edbL, 0x7efc9481L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339041e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7ffL), HEXCONSTE(-1.79273585753872790639e-03, 0xbff5, 0xeafa3bc1L, 0xca248800L), HEXCONSTE(-1.80338464113663847705e+00, 0xbfff, 0xe6d54ed3L, 0xe5310594L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339041e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7ffL), HEXCONSTE(-1.79273585753872812323e-03, 0xbff5, 0xeafa3bc1L, 0xca249000L), HEXCONSTE(-1.80338464113663850415e+00, 0xbfff, 0xe6d54ed3L, 0xe531068eL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339041e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7ffL), HEXCONSTE(-1.79273585753872803599e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc8L), HEXCONSTE(-1.80338464113663849331e+00, 0xbfff, 0xe6d54ed3L, 0xe531062aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-7.56827042671106339041e-03, 0xbff7, 0xf7ff40fcL, 0x2938b7ffL), HEXCONSTE(-1.79273585753872803610e-03, 0xbff5, 0xeafa3bc1L, 0xca248cc9L), HEXCONSTE(-1.80338464113663849331e+00, 0xbfff, 0xe6d54ed3L, 0xe531062aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398223002089532305e-01, 0x3ffe, 0xc90fdba2L, 0x2167c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(7.85398223002089421283e-01, 0x3ffe, 0xc90fdba2L, 0x2167ba35L), 0 },
	{ __LINE__, HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(7.85398223002089525366e-01, 0x3ffe, 0xc90fdba2L, 0x2167c1b5L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(7.85398103792807086951e-01, 0x3ffe, 0xc90fd9a2L, 0x2169c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(7.85398163397448198606e-01, 0x3ffe, 0xc90fdaa2L, 0x2168ba35L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(7.85398163397448302689e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c1b5L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(7.85398103792807197974e-01, 0x3ffe, 0xc90fd9a2L, 0x2169ca35L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448420651e-01, 0x3ffe, 0xc90fdaa2L, 0x2168ca35L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(7.85398163397448413712e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c9b5L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(7.85398103792807093890e-01, 0x3ffe, 0xc90fd9a2L, 0x2169c2b5L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448316567e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c2b5L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(7.85398163397448205545e-01, 0x3ffe, 0xc90fdaa2L, 0x2168bab5L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(1.00000000000000001388e+00, 0x3fff, 0x80000000L, 0x00000080L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.97733366489410400391e-01, 0x3ffd, 0x98708200L, 0x00000000L), HEXCONSTE(2.94902086257934570312e+00, 0x4000, 0xbcbcc200L, 0x00000000L), HEXCONSTE(1.00619130835590067886e-01, 0x3ffb, 0xce116722L, 0x53914e89L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.34686177141740692775e-11, 0xbfda, 0xecf14300L, 0x00000000L), HEXCONSTE(1.92693777600894122770e-10, 0x3fde, 0xd3de7a00L, 0x00000000L), HEXCONSTE(-6.97829905314132166973e-02, 0xbffb, 0x8eea6271L, 0x322bb4e9L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.33844155073165893555e-01, 0x3ffd, 0xaaed9f00L, 0x00000000L), HEXCONSTE(2.13242053985595703125e+00, 0x4000, 0x88799400L, 0x00000000L), HEXCONSTE(1.55295858732534312248e-01, 0x3ffc, 0x9f05e0a9L, 0xd785add1L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-2.60071814060211181641e-01, 0xbffd, 0x85282200L, 0x00000000L), HEXCONSTE(1.95491576194763183594e+00, 0x3fff, 0xfa3aae00L, 0x00000000L), HEXCONSTE(-1.32258195347048682869e-01, 0xbffc, 0x876eb13eL, 0x92ffadd4L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.12232970000000000000e+07, 0xc016, 0xab410100L, 0x00000000L), HEXCONSTE(-9.75658893585205078125e-01, 0xbffe, 0xf9c4c800L, 0x00000000L), HEXCONSTE(-1.57079641372647239286e+00, 0xbfff, 0xc90fdb5cL, 0xd08c644bL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.59059637784957885742e-01, 0x3ffd, 0x84a37700L, 0x00000000L), HEXCONSTE(7.25263071060180664062e+00, 0x4001, 0xe8158d00L, 0x00000000L), HEXCONSTE(3.57042232556216916333e-02, 0x3ffa, 0x923e9773L, 0x6442d916L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.77165737558791946240e+20, 0x4043, 0xa391cf00L, 0x00000000L), HEXCONSTE(3.01923007868778315776e+21, 0x4046, 0xa3ac3c00L, 0x00000000L), HEXCONSTE(1.24277371224459421689e-01, 0x3ffb, 0xfe852268L, 0x55d2207aL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-4.18084383010864257812e-01, 0xbffd, 0xd60f2800L, 0x00000000L), HEXCONSTE(3.56227183341979980469e+00, 0x4000, 0xe3fc4300L, 0x00000000L), HEXCONSTE(-1.16830070183738940311e-01, 0xbffb, 0xef449a95L, 0x0741cb1aL), FLAG_INEXACT },
	{ __LINE__, ZERO_M, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM2 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-4.11804631702911102964e-84, 0xbeea, 0x80000080L, 0x00008000L), 0 },
	{ __LINE__, HEXCONSTE(-8.02893081072683142234e-218, 0xbd2d, 0xe2bcf5e1L, 0x42f01800L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.35949084342471338272e-256, 0xbcad, 0xe2bcf6c3L, 0xffe6dc00L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(3.94310092926025390625e+00, 0x4000, 0xfc5bc400L, 0x00000000L), HEXCONSTE(-1.32959496974945068359e+00, 0xbfff, 0xaa302b00L, 0x00000000L), HEXCONSTE(1.89601858507516756217e+00, 0x3fff, 0xf2b0bcabL, 0xc0c5dd9dL), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-9.64841859740772633813e-08, 0xbfe7, 0xcf32be00L, 0x00000000L), HEXCONSTE(1.61045781338397045457e-15, 0x3fcd, 0xe8176800L, 0x00000000L), HEXCONSTE(-1.57079631010347929392e+00, 0xbfff, 0xc90fda7eL, 0x49349b6fL), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.89288349786683953756e-270, 0x3c7f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.11022302462515654042e-16, 0x3fca, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(2.86016103842627471037e-4894, 0x007f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.51100743476790622808e-4624, 0x03ff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.10714871779409050297e+00, 0x3fff, 0x8db70c97L, 0x5df22363L), 0 },
#endif
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.67755524445912377116e-4640, 0x03ca, 0x80000000L, 0x00000000L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.43008051921313735518e-4894, 0x007e, 0x80000000L, 0x00000000L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(7.55503717383953114038e-4625, 0x03fe, 0x80000000L, 0x00000000L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(4.63647609000806116202e-01, 0x3ffd, 0xed63382bL, 0x0dda7b45L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(8.38777622229561885579e-4641, 0x03c9, 0x80000000L, 0x00000000L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.70495788312977192029e-254, 0x3cb4, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489650823e+00, 0x3fff, 0xc90fdaa2L, 0x2168be35L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979312749e+00, 0x4000, 0xc90fdaa2L, 0x2168c035L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.03444393579570273543e+00, 0x4000, 0x82345456L, 0x726fb083L), 0 },
#endif
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.67794504458898712223e+00, 0x4000, 0xab63739cL, 0xbfad72ccL), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489673028e+00, 0x3fff, 0xc90fdaa2L, 0x2168c635L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.89288349786683953756e-270, 0xbc7f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.11022302462515654042e-16, 0xbfca, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.86016103842627471037e-4894, 0x807f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.51100743476790622808e-4624, 0x83ff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.10714871779409050297e+00, 0xbfff, 0x8db70c97L, 0x5df22363L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67755524445912377116e-4640, 0x83ca, 0x80000000L, 0x00000000L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.43008051921313735518e-4894, 0x807e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-7.55503717383953114038e-4625, 0x83fe, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-4.63647609000806116202e-01, 0xbffd, 0xed63382bL, 0x0dda7b45L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-8.38777622229561885579e-4641, 0x83c9, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.70495788312977192029e-254, 0xbcb4, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489650823e+00, 0xbfff, 0xc90fdaa2L, 0x2168be35L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979312749e+00, 0xc000, 0xc90fdaa2L, 0x2168c035L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.03444393579570273543e+00, 0xc000, 0x82345456L, 0x726fb083L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.67794504458898712223e+00, 0xc000, 0xab63739cL, 0xbfad72ccL), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489673028e+00, 0xbfff, 0xc90fdaa2L, 0x2168c635L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.52577026536099526576e-279, 0x3c62, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.30065065533302672992e-4906, 0x0056, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.68898299503883356464e-4628, 0x03f3, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, SUBNORM_P, HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, SUBNORM_M, HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.52577026536099526576e-279, 0xbc62, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.30065065533302672992e-4906, 0x8056, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68898299503883356464e-4628, 0x83f3, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, SUBNORM_P, HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, SUBNORM_M, HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079620758560706848e+00, 0x3fff, 0xc90fd9a2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.58786576532737530795e-263, 0x3c96, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489639721e+00, 0x3fff, 0xc90fdaa2L, 0x2168ba35L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.39927697682309554456e-4887, 0x0096, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(6.80497252017479153089e-4609, 0x0433, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661915e+00, 0x3fff, 0xc90fdaa2L, 0x2168c234L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.19963848841154777228e-4887, 0x0095, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.40248626008739576544e-4609, 0x0432, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661915e+00, 0x3fff, 0xc90fdaa2L, 0x2168c234L), FLAG_SUBNORM },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.43022233380854697687e-247, 0x3ccb, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079644600418617004e+00, 0x3fff, 0xc90fdba2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489684130e+00, 0x3fff, 0xc90fdaa2L, 0x2168ca35L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661936e+00, 0x3fff, 0xc90fdaa2L, 0x2168c236L), FLAG_SUBNORM },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079620758560706848e+00, 0xbfff, 0xc90fd9a2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.58786576532737530795e-263, 0xbc96, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489639721e+00, 0xbfff, 0xc90fdaa2L, 0x2168ba35L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.39927697682309554456e-4887, 0x8096, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-6.80497252017479153089e-4609, 0x8433, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661915e+00, 0xbfff, 0xc90fdaa2L, 0x2168c234L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.19963848841154777228e-4887, 0x8095, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40248626008739576544e-4609, 0x8432, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661915e+00, 0xbfff, 0xc90fdaa2L, 0x2168c234L), FLAG_SUBNORM },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.43022233380854697687e-247, 0xbccb, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), SUBNORM_P, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079644600418617004e+00, 0xbfff, 0xc90fdba2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489684130e+00, 0xbfff, 0xc90fdaa2L, 0x2168ca35L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661936e+00, 0xbfff, 0xc90fdaa2L, 0x2168c236L), FLAG_SUBNORM },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), SUBNORM_M, HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.19209289550780685314e-07, 0x3fe7, 0xffffffffL, 0xfffeaaabL), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(4.20304568452953727932e-286, 0x3c4b, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(2.22044604925031308085e-16, 0x3fcb, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(2.46519032881566189191e-32, 0x3f96, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.55049640576008168449e-4913, 0x003f, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(8.19118771708556528886e-4644, 0x03bf, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(5.42101086242752217004e-20, 0x3fbf, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, SUBNORM_P, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.08420217248550443401e-19, 0x3fc0, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
#endif
	{ __LINE__, SUBNORM_P, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(9.09404520253516733692e-4660, 0x038a, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159253438050368773e+00, 0x4000, 0xc90fda22L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979301647e+00, 0x4000, 0xc90fdaa2L, 0x2168be35L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, SUBNORM_P, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.14159265358979323830e+00, 0x4000, 0xc90fdaa2L, 0x2168c234L), FLAG_SUBNORM },
#endif
	{ __LINE__, SUBNORM_P, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.19209289550780685314e-07, 0xbfe7, 0xffffffffL, 0xfffeaaabL), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-4.20304568452953727932e-286, 0xbc4b, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22044604925031308085e-16, 0xbfcb, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.46519032881566189191e-32, 0xbf96, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, SUBNORM_M, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.55049640576008168449e-4913, 0x803f, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-8.19118771708556528886e-4644, 0x83bf, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-5.42101086242752217004e-20, 0xbfbf, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, SUBNORM_M, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.08420217248550443401e-19, 0xbfc0, 0x80000000L, 0x00000000L), FLAG_SUBNORM },
#endif
	{ __LINE__, SUBNORM_M, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-9.09404520253516733692e-4660, 0x838a, 0x80000000L, 0x00000000L), FLAG_SUBNORM},
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159253438050368773e+00, 0xc000, 0xc90fda22L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979301647e+00, 0xc000, 0xc90fdaa2L, 0x2168be35L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, SUBNORM_M, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323830e+00, 0xc000, 0xc90fdaa2L, 0x2168c234L), FLAG_SUBNORM },
#endif
	{ __LINE__, SUBNORM_M, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.93873605221803725101e-39, 0x3f7f, 0x80000080L, 0x00008000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.56268464626800407531e-309, 0x3bff, 0x80000000L, 0x00000400L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(8.40525785778023376566e-4933, 0x0000, 0x20000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.93873605221803725101e-39, 0xbf7f, 0x80000080L, 0x00008000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.56268464626800407531e-309, 0xbbff, 0x80000000L, 0x00000400L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-8.40525785778023376566e-4933, 0x8000, 0x20000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.45446762794009370161e-77, 0x3f01, 0x80000080L, 0x00008000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(6.53890437709391347170e-347, 0x3b81, 0x80000000L, 0x00000400L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(6.53890476684300859352e-347, 0x3b81, 0x80000080L, 0x00008000L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.23773841895303144761e-616, 0x3801, 0x80000000L, 0x00000400L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
#endif
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
#endif
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.88972181427306670596e-331, 0x3bb6, 0x80000080L, 0x00008000L), FLAG_INEXACT },
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.11485565647580385825e-600, 0x3836, 0x80000000L, 0x00000400L), FLAG_INEXACT },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.45446762794009370161e-77, 0xbf01, 0x80000080L, 0x00008000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-6.53890437709391347170e-347, 0xbb81, 0x80000000L, 0x00000400L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-6.53890476684300859352e-347, 0xbb81, 0x80000080L, 0x00008000L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.23773841895303144761e-616, 0xb801, 0x80000000L, 0x00000400L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
#endif
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.88972181427306670596e-331, 0xbbb6, 0x80000080L, 0x00008000L), 0 },
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.11485565647580385825e-600, 0xb836, 0x80000000L, 0x00000400L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.11804631702911102964e-84, 0x3eea, 0x80000080L, 0x00008000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.79498145233859237635e-354, 0x3b6a, 0x80000000L, 0x00000400L), 0 },
#endif
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.45192852559605980323e-362, 0x3b4d, 0x80000080L, 0x00008000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.74833138236958751135e-632, 0x37cd, 0x80000000L, 0x00000400L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-4.11804631702911102964e-84, 0xbeea, 0x80000080L, 0x00008000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-7.79498145233859237635e-354, 0xbb6a, 0x80000000L, 0x00000400L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.45192852559605980323e-362, 0xbb4d, 0x80000080L, 0x00008000L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.74833138236958751135e-632, 0xb7cd, 0x80000000L, 0x00000400L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
	{ __LINE__, SUBNORM_M, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_M, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_M, FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), ZERO_P, FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, SUBNORM_M, HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), ZERO_M, FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },

#if defined(TEST_LDOUBLE) && LDBL_MANT_DIG >= 64
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(2.60130131066605345983e-4906, 0x0057, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(7.37796599007766712928e-4628, 0x03f4, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(7.85398163397448309628e-01, 0x3ffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), SUBNORM_P, HEXCONSTE(1.10714871779409050297e+00, 0x3fff, 0x8db70c97L, 0x5df22363L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(4.63647609000806116202e-01, 0x3ffd, 0xed63382bL, 0x0dda7b45L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(2.35619449019234492894e+00, 0x4000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), SUBNORM_M, HEXCONSTE(2.03444393579570273543e+00, 0x4000, 0x82345456L, 0x726fb083L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(2.67794504458898712223e+00, 0x4000, 0xab63739cL, 0xbfad72ccL), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.60130131066605345983e-4906, 0x8057, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-7.37796599007766712928e-4628, 0x83f4, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-7.85398163397448309628e-01, 0xbffe, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), SUBNORM_P, HEXCONSTE(-1.10714871779409050297e+00, 0xbfff, 0x8db70c97L, 0x5df22363L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-4.63647609000806116202e-01, 0xbffd, 0xed63382bL, 0x0dda7b45L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-2.35619449019234492894e+00, 0xc000, 0x96cbe3f9L, 0x990e91a8L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), SUBNORM_M, HEXCONSTE(-2.03444393579570273543e+00, 0xc000, 0x82345456L, 0x726fb083L), FLAG_SUBNORM },
	{ __LINE__, SUBNORM_M, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-2.67794504458898712223e+00, 0xc000, 0xab63739cL, 0xbfad72ccL), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661915e+00, 0x3fff, 0xc90fdaa2L, 0x2168c234L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661904e+00, 0x3fff, 0xc90fdaa2L, 0x2168c233L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661936e+00, 0x3fff, 0xc90fdaa2L, 0x2168c236L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661947e+00, 0x3fff, 0xc90fdaa2L, 0x2168c237L), 0 },
#endif
	{ __LINE__, HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.57079632679489661926e+00, 0x3fff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661915e+00, 0xbfff, 0xc90fdaa2L, 0x2168c234L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661904e+00, 0xbfff, 0xc90fdaa2L, 0x2168c233L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661936e+00, 0xbfff, 0xc90fdaa2L, 0x2168c236L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661947e+00, 0xbfff, 0xc90fdaa2L, 0x2168c237L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.57079632679489661926e+00, 0xbfff, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(3.10099281152016336898e-4913, 0x0040, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(1.63823754341711305777e-4643, 0x03c0, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(1.08420217248550443401e-19, 0x3fc0, 0x80000000L, 0x00000000L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(2.16840434497100886801e-19, 0x3fc1, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(1.81880904050703346738e-4659, 0x038b, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323830e+00, 0x4000, 0xc90fdaa2L, 0x2168c234L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.14159265358979323830e+00, 0x4000, 0xc90fdaa2L, 0x2168c234L), 0 },
#endif
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L) },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.10099281152016336898e-4913, 0x8040, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(2.22507385850720138309e-308, 0x3c01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63823754341711305777e-4643, 0x83c0, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(3.36210314311209350626e-4932, 0x0001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.08420217248550443401e-19, 0xbfc0, 0x80000000L, 0x00000000L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.68105157155604675313e-4932, 0x0000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.16840434497100886801e-19, 0xbfc1, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(2.00416836000897277800e-292, 0x3c36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.81880904050703346738e-4659, 0x838b, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323830e+00, 0xc000, 0xc90fdaa2L, 0x2168c234L), 0 },
#if defined (TEST_LDOUBLE) && __LDBL_MIN_EXP__ <= (-16382)
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323830e+00, 0xc000, 0xc90fdaa2L, 0x2168c234L), 0 },
#endif
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, SUBNORM_P, HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), FLAG_SUBNORM },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_P, 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), 0 },
	{ __LINE__, HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.14159265358979323851e+00, 0x4000, 0xc90fdaa2L, 0x2168c235L), 0 },
	{ __LINE__, HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.14159265358979323851e+00, 0xc000, 0xc90fdaa2L, 0x2168c235L), 0 },
#endif
};

int main(int argc, char **argv)
{
	int status;
	
	status = 0;
	
	test_init(argc, argv);
	
	status |= test_table_ff_f(fatan2_data, ARRAY_SIZE(fatan2_data), __FILE__);
	
	return status ? EXIT_FAILURE : EXIT_SUCCESS;
}
