#include "testconfig.h"
#define TEST_FUNC_FF_F __MATH_PRECNAME(pow)
#include "testdriver.h"


static test_ff_f_data const fpow_data[] = {
	/* If y is 0, the result is 1.0 (even if x is a NaN). */
	{ __LINE__,  QNAN_P, ZERO_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  QNAN_M, ZERO_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  QNAN_P, ZERO_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  QNAN_M, ZERO_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  ZERO_P, ZERO_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  ZERO_M, ZERO_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  ZERO_P, ZERO_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  ZERO_M, ZERO_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */

	{ __LINE__,  SNAN_P, ZERO_P, QNAN_P, FLAG_FAIL_SOFTFLOAT },
	{ __LINE__,  SNAN_M, ZERO_P, QNAN_P, FLAG_FAIL_SOFTFLOAT },
	{ __LINE__,  SNAN_P, ZERO_M, QNAN_P, FLAG_FAIL_SOFTFLOAT },
	{ __LINE__,  SNAN_M, ZERO_M, QNAN_P, FLAG_FAIL_SOFTFLOAT },
	
	/* If the absolute value of x is greater than 1, and y is positive infinity, the result is positive infinity. */
	{ __LINE__,  HEXCONSTE(1.1L, 0x3fff, 0x8cccccccL, 0xcccccccdL), INF_P, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_P, INF_P, INF_P, FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  HEXCONSTE(-1.1L, 0xbfff, 0x8cccccccL, 0xcccccccdL), INF_P, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, INF_P, INF_P, FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	
	/* If the absolute value of x is less than 1, and y is positive infinity, the result is +0. */
	{ __LINE__,  HEXCONSTE(0.9L, 0x3ffe, 0xe6666666L, 0x66666666L), INF_P, ZERO_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  HEXCONSTE(1e-7L, 0x3fe7, 0xd6bf94d5L, 0xe57a42bcL), INF_P, ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  HEXCONSTE(-0.9L, 0xbffe, 0xe6666666L, 0x66666666L), INF_P, ZERO_P, FLAG_IGNORE_ZEROSIGN|FLAG_FAIL_ARANYM },
	{ __LINE__,  HEXCONSTE(-1e-7L, 0xbfe7, 0xd6bf94d5L, 0xe57a42bcL), INF_P, ZERO_P, FLAG_IGNORE_ZEROSIGN|FLAG_FAIL_ARANYM },
	
	/* If the absolute value of x is greater than 1, and y is negative infinity, the result is +0. */
	{ __LINE__,  HEXCONSTE(1.1L, 0x3fff, 0x8cccccccL, 0xcccccccdL), INF_M, ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, INF_M, ZERO_P, FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  HEXCONSTE(-1.1L, 0xbfff, 0x8cccccccL, 0xcccccccdL), INF_M, ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, INF_M, ZERO_P, FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	
	/* If the absolute value of x is less than 1, and y is negative infinity, the result is positive infinity. */
	{ __LINE__,  HEXCONSTE(0.9L, 0x3ffe, 0xe6666666L, 0x66666666L), INF_M, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  HEXCONSTE(1e-7L, 0x3fe7, 0xd6bf94d5L, 0xe57a42bcL), INF_M, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  HEXCONSTE(-0.9L, 0xbffe, 0xe6666666L, 0x66666666L), INF_M, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  HEXCONSTE(-1e-7L, 0xbfe7, 0xd6bf94d5L, 0xe57a42bcL), INF_M, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	
	/* If x is positive infinity, and y greater than 0, the result is positive infinity. */
	{ __LINE__,  INF_P, HEXCONSTE(1e-7L, 0x3fe7, 0xd6bf94d5L, 0xe57a42bcL), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_P, HEXCONSTE(1e7L, 0x4016, 0x98968000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_P, SUBNORM_P, INF_P, FLAG_SUBNORM },
	
	/* If x is positive infinity, and y less than 0, the result is +0. */
	{ __LINE__,  INF_P, HEXCONSTE(-1e-7L, 0xbfe7, 0xd6bf94d5L, 0xe57a42bcL), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-1e7L, 0xc016, 0x98968000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, SUBNORM_M, ZERO_P, FLAG_SUBNORM },
	
	/* If x is negative infinity, and y is an odd integer greater than 0, the result is negative infinity. */
	{ __LINE__,  INF_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(11, 0x4002, 0xb0000000L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(1001, 0x4008, 0xfa400000L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM }, /* returns zero */
	
	/* If x is negative infinity, and y greater than 0 and not an odd integer, the result is positive infinity. */
	{ __LINE__,  INF_M, HEXCONSTE(2, 0x4000, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(12, 0x4002, 0xc0000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(1002, 0x4008, 0xfa800000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_M, HEXCONSTE(0.1L, 0x3ffb, 0xccccccccL, 0xcccccccdL), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(1.1L, 0x3fff, 0x8cccccccL, 0xcccccccdL), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(11.1L, 0x4002, 0xb1999999L, 0x9999999aL), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
	{ __LINE__,  INF_M, HEXCONSTE(1001.1L, 0x4008, 0xfa466666L, 0x66666666L), INF_P, FLAG_FAIL_ARANYM }, /* returns zero */
#endif
	{ __LINE__,  INF_M, SUBNORM_P, INF_P, FLAG_SUBNORM },
	
	/* If x is negative infinity, and y is an odd integer less than 0, the result is -0. */
	{ __LINE__,  INF_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), ZERO_M, FLAG_FAIL_SOFTFLOAT|FLAG_FAIL_ARANYM }, /* returns +zero */
	{ __LINE__,  INF_M, HEXCONSTE(-11, 0xc002, 0xb0000000L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  INF_M, HEXCONSTE(-1001, 0xc008, 0xfa400000L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM }, /* returns NaN */

	/* If x is negative infinity, and y less than 0 and not an odd integer, the result is +0. */
	{ __LINE__,  INF_M, HEXCONSTE(-2, 0xc000, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, HEXCONSTE(-12, 0xc002, 0xc0000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, HEXCONSTE(-1002, 0xc008, 0xfa800000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_M, HEXCONSTE(-0.1L, 0xbffb, 0xccccccccL, 0xcccccccdL), ZERO_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  INF_M, HEXCONSTE(-1.1L, 0xbfff, 0x8cccccccL, 0xcccccccdL), ZERO_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  INF_M, HEXCONSTE(-11.1L, 0xc002, 0xb1999999L, 0x9999999aL), ZERO_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  INF_M, HEXCONSTE(-1001.1L, 0xc008, 0xfa466666L, 0x66666666L), ZERO_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  INF_M, SUBNORM_M, ZERO_P, FLAG_SUBNORM },
#endif

	{ __LINE__,  QNAN_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  QNAN_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  QNAN_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  QNAN_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  QNAN_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  QNAN_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__,  QNAN_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  QNAN_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__,  SNAN_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  SNAN_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  SNAN_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  SNAN_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  SNAN_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  SNAN_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__,  SNAN_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  SNAN_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__,  ZERO_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  ZERO_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  ZERO_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  ZERO_P, SNAN_M, QNAN_P, 0 },
	
	/* If x is +1, the result is 1.0 (even if y is a NaN) */
	{ __LINE__,  HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), SNAN_P, QNAN_P, FLAG_FAIL_SOFTFLOAT },
	{ __LINE__,  HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), SNAN_M, QNAN_P, FLAG_FAIL_SOFTFLOAT },

	{ __LINE__,  HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_M, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), SNAN_P, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), SNAN_M, QNAN_P, 0 },
	{ __LINE__,  QNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), QNAN_P, 0 },

	/*  pow (x, qNaN or sNaN) == qNaN.  */
	{ __LINE__,  HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_M, QNAN_P, 0 },
	{ __LINE__,  ZERO_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  ZERO_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  INF_P, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  INF_P, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), QNAN_P, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), QNAN_M, QNAN_P, 0 },
	{ __LINE__,  INF_M, QNAN_P, QNAN_P, 0 },
	{ __LINE__,  INF_M, QNAN_M, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), SNAN_P, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), SNAN_M, QNAN_P, 0 },
	{ __LINE__,  ZERO_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  ZERO_M, SNAN_M, QNAN_P, 0 },
	{ __LINE__,  INF_P, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  INF_P, SNAN_M, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), SNAN_P, QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), SNAN_M, QNAN_P, 0 },
	{ __LINE__,  INF_M, SNAN_P, QNAN_P, 0 },
	{ __LINE__,  INF_M, SNAN_M, QNAN_P, 0 },

	{ __LINE__,  QNAN_P, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_M, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_P, HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_M, HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_P, INF_P, QNAN_P, 0 },
	{ __LINE__,  QNAN_M, INF_P, QNAN_P, 0 },
	{ __LINE__,  QNAN_P, INF_M, QNAN_P, 0 },
	{ __LINE__,  QNAN_M, INF_M, QNAN_P, 0 },
	{ __LINE__,  QNAN_P, HEXCONSTE(2.5, 0x4000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_M, HEXCONSTE(2.5, 0x4000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_P, HEXCONSTE(-2.5, 0xc000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_M, HEXCONSTE(-2.5, 0xc000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  QNAN_P, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  QNAN_M, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  QNAN_P, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  QNAN_M, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  SNAN_P, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_M, HEXCONSTE(3.0, 0x4000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_P, HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_M, HEXCONSTE(-3.0, 0xc000, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_P, INF_P, QNAN_P, 0 },
	{ __LINE__,  SNAN_M, INF_P, QNAN_P, 0 },
	{ __LINE__,  SNAN_P, INF_M, QNAN_P, 0 },
	{ __LINE__,  SNAN_M, INF_M, QNAN_P, 0 },
	{ __LINE__,  SNAN_P, HEXCONSTE(2.5, 0x4000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_M, HEXCONSTE(2.5, 0x4000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_P, HEXCONSTE(-2.5, 0xc000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_M, HEXCONSTE(-2.5, 0xc000, 0xa0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  SNAN_P, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  SNAN_M, SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  SNAN_P, SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  SNAN_M, SUBNORM_M, QNAN_P, FLAG_SUBNORM },

	{ __LINE__,  HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), INF_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_SOFTFLOAT|FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), INF_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_SOFTFLOAT|FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), INF_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_SOFTFLOAT|FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), INF_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_SOFTFLOAT|FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* returns NaN */

	/*  pow (x, +-0) == 1.  */
	{ __LINE__,  INF_P, ZERO_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  INF_P, ZERO_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  INF_M, ZERO_P, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */
	{ __LINE__,  INF_M, ZERO_M, HEXCONSTE(1, 0x3fff, 0x80000000L, 0x00000000L), FLAG_FAIL_HARDFLOAT|FLAG_FAIL_ARANYM }, /* glibc returns NaN */

	{ __LINE__,  HEXCONSTE(-0.1L, 0xbffb, 0xccccccccL, 0xcccccccdL), HEXCONSTE(1.1L, 0x3fff, 0x8cccccccL, 0xcccccccdL), QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-0.1L, 0xbffb, 0xccccccccL, 0xcccccccdL), HEXCONSTE(-1.1L, 0xbfff, 0x8cccccccL, 0xcccccccdL), QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-10.1L, 0xc002, 0xa1999999L, 0x9999999aL), HEXCONSTE(1.1L, 0x3fff, 0x8cccccccL, 0xcccccccdL), QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-10.1L, 0xc002, 0xa1999999L, 0x9999999aL), HEXCONSTE(-1.1L, 0xbfff, 0x8cccccccL, 0xcccccccdL), QNAN_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.01L, 0xbfff, 0x8147ae14L, 0x7ae147aeL), SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  HEXCONSTE(-1.01L, 0xbfff, 0x8147ae14L, 0x7ae147aeL), SUBNORM_M, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  HEXCONSTE(-1.0L, 0xbfff, 0x80000000L, 0x00000000L), SUBNORM_P, QNAN_P, FLAG_SUBNORM },
	{ __LINE__,  HEXCONSTE(-1.0L, 0xbfff, 0x80000000L, 0x00000000L), SUBNORM_M, QNAN_P, FLAG_SUBNORM },

#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  ZERO_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_P, HEXCONSTE(-11, 0xc002, 0xb0000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_P, HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_P, HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_P, HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-11L, 0xc002, 0xb0000000L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
#endif

#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  ZERO_P, HEXCONSTE(-2, 0xc000, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_P, HEXCONSTE(-11.1L, 0xc002, 0xb1999999L, 0x9999999aL), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
#endif
	{ __LINE__,  ZERO_P, SUBNORM_M, INF_P, FLAG_SUBNORM },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  ZERO_P, HEXCONSTE(-1.67772160000000000000e+07, 0xc017, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_P, HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
#endif
	{ __LINE__,  ZERO_P, MAX_M, INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  ZERO_M, HEXCONSTE(-2, 0xc000, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-11.1L, 0xc002, 0xb1999999L, 0x9999999aL), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
#endif
	{ __LINE__,  ZERO_M, SUBNORM_M, INF_P, FLAG_SUBNORM },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  ZERO_M, HEXCONSTE(-1.67772160000000000000e+07, 0xc017, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
	{ __LINE__,  ZERO_M, HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM }, /* returns NaN */
#endif
	{ __LINE__,  ZERO_M, MAX_M, INF_P, 0 },

	/* If the absolute value of x is less than 1, and y is positive infinity, the result is +0 */
	{ __LINE__,  ZERO_P, INF_P, ZERO_P, 0 },
	{ __LINE__,  ZERO_M, INF_P, ZERO_P, 0 },
	
	/* If the absolute value of x is less than 1, and y is negative infinity, the result is positive infinity. */
	{ __LINE__,  ZERO_P, INF_M, INF_P, FLAG_FAIL_ARANYM|FLAG_FAIL_HARDFLOAT }, /* glibc returns NaN */
	{ __LINE__,  ZERO_M, INF_M, INF_P, FLAG_FAIL_ARANYM|FLAG_FAIL_HARDFLOAT }, /* glibc returns NaN */

	/*  pow (x, +inf) == +inf for |x| > 1.  */
	{ __LINE__,  HEXCONSTE(1.5, 0x3fff, 0xc0000000L, 0x00000000L), INF_P, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */

	/*  pow (x, +inf) == +0 for |x| < 1.  */
	{ __LINE__,  HEXCONSTE(0.5, 0x3ffe, 0x80000000L, 0x00000000L), INF_P, ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */

	/*  pow (x, -inf) == +0 for |x| > 1.  */
	{ __LINE__,  HEXCONSTE(1.5, 0x3fff, 0xc0000000L, 0x00000000L), INF_M, ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */

	/*  pow (x, -inf) == +inf for |x| < 1.  */
	{ __LINE__,  HEXCONSTE(0.5, 0x3ffe, 0x80000000L, 0x00000000L), INF_M, INF_P, FLAG_FAIL_ARANYM }, /* returns zero */

	/*  pow (+inf, y) == +inf for y > 0.  */
	{ __LINE__,  INF_P, HEXCONSTE(2, 0x4000, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_P, HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_P, HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_P, HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_P, HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_P, HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), INF_P, 0 },
#endif
	{ __LINE__,  INF_P, MAX_P, INF_P, 0 },

	/*  pow (+inf, y) == +0 for y < 0.  */
	{ __LINE__,  INF_P, HEXCONSTE(-1, 0xbfff, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-1.67772160000000000000e+07, 0xc017, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_P, MAX_M, ZERO_P, 0 },

	/*  pow (-inf, y) == -inf for y an odd integer > 0.  */
	{ __LINE__,  INF_M, HEXCONSTE(27, 0x4003, 0xd8000000L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), INF_M, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), INF_M, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), INF_M, FLAG_FAIL_ARANYM },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_M, HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif

	/*  pow (-inf, y) == +inf for y > 0 and not an odd integer.  */
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_M, HEXCONSTE(0.5, 0x3ffe, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(28, 0x4003, 0xe0000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), INF_P, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), INF_P, 0 },
#endif
	{ __LINE__,  INF_M, MAX_P, INF_P, 0 },

	/*  pow (-inf, y) == -0 for y an odd integer < 0. */
	{ __LINE__,  INF_M, HEXCONSTE(-3, 0xc000, 0xc0000000L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM },
	{ __LINE__,  INF_M, HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), ZERO_M, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_M, HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), ZERO_M, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
#endif
	{ __LINE__,  INF_M, HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  INF_M, HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), ZERO_M, 0 },
#endif
	{ __LINE__,  INF_M, HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */

	/*  pow (-inf, y) == +0 for y < 0 and not an odd integer.  */
	{ __LINE__,  INF_M, HEXCONSTE(-2.0, 0xc000, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, HEXCONSTE(-1.67772160000000000000e+07, 0xc017, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), ZERO_P, FLAG_FAIL_ARANYM }, /* inf treated as NaN? */
	{ __LINE__,  INF_M, MAX_M, ZERO_P, 0 },

	{ __LINE__,  MAX_M, HEXCONSTE(0.5, 0x3ffe, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  MAX_M, HEXCONSTE(1.5, 0x3fff, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  MAX_M, HEXCONSTE(1000.5, 0x4008, 0xfa200000L, 0x00000000L), QNAN_P, 0 },

	{ __LINE__,  MIN_M, HEXCONSTE(0.5, 0x3ffe, 0x80000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  MIN_M, HEXCONSTE(1.5, 0x3fff, 0xc0000000L, 0x00000000L), QNAN_P, 0 },
	{ __LINE__,  MIN_M, HEXCONSTE(1000.5, 0x4008, 0xfa200000L, 0x00000000L), QNAN_P, 0 },

	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+01, 0xc002, 0xa0000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+01, 0xc002, 0xa0000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.25000000000000000000e+00, 0x3fff, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.25000000000000000000e+00, 0xbfff, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.61168601842738790400e+18, 0x403d, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.22337203685477580800e+18, 0x403e, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.72236648286964521370e+21, 0x4047, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.27500000000000000000e+01, 0x4004, 0x83000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.27500000000000000000e+01, 0x4004, 0x83000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.27500000000000000000e+01, 0xc004, 0x83000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.27500000000000000000e+01, 0xc004, 0x83000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.72236648286964521370e+21, 0x4047, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.72236648286964521370e+21, 0x4047, 0x80000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(2.11758236813575084767e-22, 0x3fb7, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(2.11758236813575084767e-22, 0x3fb7, 0x80000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.72236648286964521370e+21, 0x4047, 0x80000000L, 0x00000000L), HEXCONSTE(4.72236648286964521370e+21, 0x4047, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.72236648286964521370e+21, 0xc047, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10000000000000000000e+01, 0x4002, 0xb0000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10000000000000000000e+01, 0x4002, 0xb0000000L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.11000003814697265625e+01, 0x4002, 0xb1999a00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10999994277954101562e+01, 0x4002, 0xb1999900L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.11000000000000014211e+01, 0x4002, 0xb1999999L, 0x9999a000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10999999999999996447e+01, 0x4002, 0xb1999999L, 0x99999800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.11000000000000000003e+01, 0x4002, 0xb1999999L, 0x9999999aL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10999999999999999995e+01, 0x4002, 0xb1999999L, 0x99999999L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.11000003814697265625e+01, 0x4002, 0xb1999a00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10999994277954101562e+01, 0x4002, 0xb1999900L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.11000000000000014211e+01, 0x4002, 0xb1999999L, 0x9999a000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10999999999999996447e+01, 0x4002, 0xb1999999L, 0x99999800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.11000000000000000003e+01, 0x4002, 0xb1999999L, 0x9999999aL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.10999999999999999995e+01, 0x4002, 0xb1999999L, 0x99999999L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(2.70000000000000000000e+01, 0x4003, 0xd8000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(2.70000000000000000000e+01, 0x4003, 0xd8000000L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.70141183460469231732e+38, 0x407e, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.60000000000000000000e+01, 0x4003, 0x80000000L, 0x00000000L), HEXCONSTE(2.50000000000000000000e-01, 0x3ffd, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(1.25000000000000000000e-01, 0x3ffc, 0x80000000L, 0x00000000L), HEXCONSTE(2.56000000000000000000e+02, 0x4007, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.60000000000000000000e+01, 0x4003, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(2.56000000000000000000e+02, 0x4007, 0x80000000L, 0x00000000L), HEXCONSTE(8.00000000000000000000e+00, 0x4002, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.25000000000000000000e+00, 0x3fff, 0xa0000000L, 0x00000000L), HEXCONSTE(6.97953644326574699195e-01, 0x3ffe, 0xb2ad170cL, 0x81b4183dL), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.80817973168046080000e+16, 0xc037, 0xae3a7000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.80818059067392000000e+16, 0xc037, 0xae3a7100L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.80818000000000000000e+16, 0xc037, 0xae3a704fL, 0xf7280000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49320999999999959167e+133, 0xc1bb, 0xd322818aL, 0xade07800L), HEXCONSTE(-9.80817973168046080000e+16, 0xc037, 0xae3a7000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49320999999999959167e+133, 0xc1bb, 0xd322818aL, 0xade07800L), HEXCONSTE(-9.80818059067392000000e+16, 0xc037, 0xae3a7100L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49320999999999959167e+133, 0xc1bb, 0xd322818aL, 0xade07800L), HEXCONSTE(-9.80818000000000000000e+16, 0xc037, 0xae3a704fL, 0xf7280000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49321000000000060036e+133, 0xc1bb, 0xd322818aL, 0xade08000L), HEXCONSTE(-9.80817973168046080000e+16, 0xc037, 0xae3a7000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49321000000000060036e+133, 0xc1bb, 0xd322818aL, 0xade08000L), HEXCONSTE(-9.80818059067392000000e+16, 0xc037, 0xae3a7100L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49321000000000060036e+133, 0xc1bb, 0xd322818aL, 0xade08000L), HEXCONSTE(-9.80818000000000000000e+16, 0xc037, 0xae3a704fL, 0xf7280000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49320999999999999997e+133, 0xc1bb, 0xd322818aL, 0xade07b3dL), HEXCONSTE(-9.80817973168046080000e+16, 0xc037, 0xae3a7000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49320999999999999997e+133, 0xc1bb, 0xd322818aL, 0xade07b3dL), HEXCONSTE(-9.80818059067392000000e+16, 0xc037, 0xae3a7100L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49320999999999999997e+133, 0xc1bb, 0xd322818aL, 0xade07b3dL), HEXCONSTE(-9.80818000000000000000e+16, 0xc037, 0xae3a704fL, 0xf7280000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49321000000000000046e+133, 0xc1bb, 0xd322818aL, 0xade07b3eL), HEXCONSTE(-9.80817973168046080000e+16, 0xc037, 0xae3a7000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49321000000000000046e+133, 0xc1bb, 0xd322818aL, 0xade07b3eL), HEXCONSTE(-9.80818059067392000000e+16, 0xc037, 0xae3a7100L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-7.49321000000000000046e+133, 0xc1bb, 0xd322818aL, 0xade07b3eL), HEXCONSTE(-9.80818000000000000000e+16, 0xc037, 0xae3a704fL, 0xf7280000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(8.50705917302346158658e+37, 0x407d, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.27000000000000000000e+02, 0x4005, 0xfe000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.27000000000000000000e+02, 0xc005, 0xfe000000L, 0x00000000L), HEXCONSTE(-5.87747175411143753984e-39, 0xbf80, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(8.63616958460605456449e-78, 0x3eff, 0x80000100L, 0x00018000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(3.09434604738257896256e-617, 0x37ff, 0x80000000L, 0x00000800L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-2.53794229113506834416e-116, 0xbe7f, 0x80000180L, 0x00030000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.72128712480151578340e-925, 0xb3ff, 0x80000000L, 0x00000c00L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.15792075433823913219e+77, 0x40fe, 0xfffffe00L, 0x00010000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(3.23170060713110001249e+616, 0x47fe, 0xffffffffL, 0xfffff000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.94019991507671508429e+115, 0xc17e, 0xfffffd00L, 0x00030000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-5.80960599536995612787e+924, 0xcbfe, 0xffffffffL, 0xffffe800L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), INF_M, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.36112946768375385385e+39, 0x4081, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-5.02168138830934461107e+58, 0xc0c2, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.85267342779705912678e+78, 0x4103, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-6.83515851494691226366e+97, 0xc144, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-8.79609302220800000000e+12, 0xc02a, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-6.80564733841876926927e+38, 0xc080, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-8.79609302220800000000e+12, 0xc02a, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(5.98631070650737835296e+51, 0x40ab, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-8.79609302220800000000e+12, 0xc02a, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-5.26561458342785933490e+64, 0xc0d6, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-8.58993459200000000000e+09, 0xc020, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(5.44451787073501541541e+39, 0x4083, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-8.58993459200000000000e+09, 0xc020, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.67680523945888933825e+49, 0xc0a4, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-6.71088640000000000000e+07, 0xc019, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.36112946768375385385e+39, 0xc081, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.71050543121376108502e-20, 0xbfbe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.36112946768375385385e+39, 0x4081, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.71050543121376108502e-20, 0xbfbe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-5.02168138830934461107e+58, 0xc0c2, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.71050543121376108502e-20, 0xbfbe, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.85267342779705912678e+78, 0x4103, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.71050543121376108502e-20, 0xbfbe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-6.83515851494691226366e+97, 0xc144, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.13686837721616029739e-13, 0xbfd4, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-6.80564733841876926927e+38, 0xc080, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.13686837721616029739e-13, 0xbfd4, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(5.98631070650737835296e+51, 0x40ab, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.13686837721616029739e-13, 0xbfd4, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-5.26561458342785933490e+64, 0xc0d6, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.16415321826934814453e-10, 0xbfde, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(5.44451787073501541541e+39, 0x4083, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.16415321826934814453e-10, 0xbfde, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.67680523945888933825e+49, 0xc0a4, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.49011611938476562500e-08, 0xbfe5, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.36112946768375385385e+39, 0xc081, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.15792075433823913219e+77, 0x40fe, 0xfffffe00L, 0x00010000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.68156158598851941991e+154, 0xc200, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(7.19077253944926363092e+308, 0x4401, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.94019991507671508429e+115, 0xc17e, 0xfffffd00L, 0x00030000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.68156158598851941991e+154, 0xc200, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.92824994153682607086e+463, 0xc602, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.34078047332723673806e+154, 0x41fe, 0xfffffc00L, 0x00060000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.68156158598851941991e+154, 0xc200, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(5.17072097140976116811e+617, 0x4803, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.68156158598851941991e+154, 0xc200, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.38656067287976569376e+772, 0xca04, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.94019991507671508429e+115, 0xc17e, 0xfffffd00L, 0x00030000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-8.95897896871121684223e+102, 0xc155, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-7.19077253944926363092e+308, 0xc401, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.34078047332723673806e+154, 0x41fe, 0xfffffc00L, 0x00060000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-8.95897896871121684223e+102, 0xc155, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(6.44219799497121017112e+411, 0x4557, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-8.95897896871121684223e+102, 0xc155, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-5.77155163492206414036e+514, 0xc6ad, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.34078047332723673806e+154, 0x41fe, 0xfffffc00L, 0x00060000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.31584178474632390847e+77, 0xc100, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(2.87630901577970545237e+309, 0x4403, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.31584178474632390847e+77, 0xc100, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-6.66107660458521541243e+386, 0xc504, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-5.14220174162876888173e+61, 0xc0cc, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.59538626972463181546e+308, 0xc400, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(5.09258994083621521567e+89, 0x4129, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.72917036560010337165e-155, 0xbdfe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(7.19077253944926363092e+308, 0x4401, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.63419362147803445275e+134, 0xc1be, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.72917036560010337165e-155, 0xbdfe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.92824994153682607086e+463, 0xc602, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.59344723055062059907e+179, 0x4253, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.72917036560010337165e-155, 0xbdfe, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(5.17072097140976116811e+617, 0x4803, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.72917036560010337165e-155, 0xbdfe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.38656067287976569376e+772, 0xca04, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.63419362147803445275e+134, 0xc1be, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.11619862429909666232e-103, 0xbea9, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-7.19077253944926363092e+308, 0xc401, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.59344723055062059907e+179, 0x4253, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.11619862429909666232e-103, 0xbea9, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(6.44219799497121017112e+411, 0x4557, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.11619862429909666232e-103, 0xbea9, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-5.77155163492206414036e+514, 0xc6ad, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.59344723055062059907e+179, 0x4253, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-4.31808427754722231269e-78, 0xbefe, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.87630901577970545237e+309, 0x4403, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-4.31808427754722231269e-78, 0xbefe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-6.66107660458521541243e+386, 0xc504, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.94469227433160678348e-62, 0xbf32, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.59538626972463181546e+308, 0xc400, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.15792075433823913219e+77, 0x40fe, 0xfffffe00L, 0x00010000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(3.23170060713110001249e+616, 0x47fe, 0xffffffffL, 0xfffff000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.09074813561941592946e+2466, 0xdfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.94019991507671508429e+115, 0xc17e, 0xfffffd00L, 0x00030000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-5.80960599536995612787e+924, 0xcbfe, 0xffffffffL, 0xffffe800L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.09074813561941592946e+2466, 0xdfff, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.34078047332723673806e+154, 0x41fe, 0xfffffc00L, 0x00060000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.04438888141315204289e+1233, 0x4ffe, 0xffffffffL, 0xffffe000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.09074813561941592946e+2466, 0xdfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.87749072224295658266e+1541, 0xd3fe, 0xffffffffL, 0xffffd800L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.09074813561941592946e+2466, 0xdfff, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.94019991507671508429e+115, 0xc17e, 0xfffffd00L, 0x00030000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-5.80960599536995612787e+924, 0xcbfe, 0xffffffffL, 0xffffe800L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.68203998945250901701e+1644, 0xd555, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.34078047332723673806e+154, 0x41fe, 0xfffffc00L, 0x00060000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.04438888141315204289e+1233, 0x4ffe, 0xffffffffL, 0xffffe000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.68203998945250901701e+1644, 0xd555, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.87749072224295658266e+1541, 0xd3fe, 0xffffffffL, 0xffffd800L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.68203998945250901701e+1644, 0xd555, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.34078047332723673806e+154, 0x41fe, 0xfffffc00L, 0x00060000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.04438888141315204289e+1233, 0x4ffe, 0xffffffffL, 0xffffe000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.08877776282630501338e+1233, 0xd000, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.87749072224295658266e+1541, 0xd3fe, 0xffffffffL, 0xffffd800L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.08877776282630501338e+1233, 0xd000, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.56243925790909569817e+192, 0xc27e, 0xfffffb00L, 0x000a0000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.87749072224295658266e+1541, 0xd3fe, 0xffffffffL, 0xffffd800L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.98741660675683357550e+986, 0xcccc, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), HEXCONSTE(2.57000000000000000000e+02, 0x4007, 0x80800000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(5.09258994083621521567e+89, 0x4129, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(4.09666721438767321452e+646, 0x4863, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.16801933777423582811e-2467, 0x9fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.63419362147803445275e+134, 0xc1be, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-8.29174675242248408884e+969, 0xcc95, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.16801933777423582811e-2467, 0x9fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.59344723055062059907e+179, 0x4253, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.67826822654388580915e+1293, 0x50c7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.16801933777423582811e-2467, 0x9fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.39685270706546537607e+1616, 0xd4f9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.16801933777423582811e-2467, 0x9fff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-3.63419362147803445275e+134, 0xc1be, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-8.29174675242248408884e+969, 0xcc95, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.94516186458499272259e-1645, 0xaaa9, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.59344723055062059907e+179, 0x4253, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.67826822654388580915e+1293, 0x50c7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.94516186458499272259e-1645, 0xaaa9, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.39685270706546537607e+1616, 0xd4f9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.94516186458499272259e-1645, 0xaaa9, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(2.59344723055062059907e+179, 0x4253, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.67826822654388580915e+1293, 0x50c7, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.78748873047609267897e-1234, 0xaffe, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.39685270706546537607e+1616, 0xd4f9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.78748873047609267897e-1234, 0xaffe, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.85074578797901741880e+224, 0xc2e8, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.39685270706546537607e+1616, 0xd4f9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.34737377350797089948e-987, 0xb332, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-5.42101086242752217004e-20, 0xbfbf, 0x80000000L, 0x00000000L), HEXCONSTE(-2.57000000000000000000e+02, 0xc007, 0x80800000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17549435082228750797e-38, 0x3f81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.27000000000000000000e+02, 0x4005, 0xfe000000L, 0x00000000L), HEXCONSTE(-5.87747175411143753984e-39, 0xbf80, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(8.50705917302346158658e+37, 0x407d, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.27000000000000000000e+02, 0xc005, 0xfe000000L, 0x00000000L), HEXCONSTE(-1.70141183460469231732e+38, 0xc07e, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-5.00000000000000000000e-01, 0xbffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(7.23700557733226221397e+75, 0x40fb, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(2.01981287945693795629e+615, 0x47fb, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(2.48961158822727886846e+583, 0x4791, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-6.15656346818663737692e+113, 0xc179, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-9.07750936776555947322e+922, 0xcbf9, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.24221679071718941465e+875, 0xcb5a, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.38178696881511114006e-76, 0x3f03, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(4.95095367581212524077e-616, 0x3803, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(4.01669081526105551552e-584, 0x386d, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.62428277588201550290e-114, 0xbe85, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-1.10162375987296973446e-923, 0xb405, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(-8.05012464388485347938e-876, 0xb4a4, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.67772150000000000000e+07, 0xc016, 0xffffff00L, 0x00000000L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.35544300000000000000e+07, 0xc017, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719871787008000000e+15, 0xc033, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099200000e+15, 0xc034, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-9.00719925474099100000e+15, 0xc033, 0xffffffffL, 0xfffff800L), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143974357401600000e+16, 0xc034, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819840000e+16, 0xc035, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.80143985094819820000e+16, 0xc034, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467429741979238400e+19, 0xc03e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516160e+19, 0xc03f, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095495680e+19, 0xc03e, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.84467440737095516150e+19, 0xc03e, 0xffffffffL, 0xffffffffL), INF_M, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934859483958476800e+19, 0xc03f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032320e+19, 0xc040, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474190991360e+19, 0xc03f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.68934881474191032300e+19, 0xc03f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296335789034032373e+31, 0xc068, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816958e+31, 0xc069, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066726886e+31, 0xc068, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-8.11296384146066816914e+31, 0xc068, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259267157806806475e+32, 0xc069, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363392e+32, 0xc06a, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213345377e+32, 0xc069, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.62259276829213363383e+32, 0xc069, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845930980996356144e+34, 0xc06f, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552571e+34, 0xc070, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696541041e+34, 0xc06f, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.03845937170696552565e+34, 0xc06f, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691861961992712287e+34, 0xc070, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105141e+34, 0xc071, 0x80000000L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393082083e+34, 0xc070, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-2.07691874341393105130e+34, 0xc070, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-3.40282346638528859812e+38, 0xc07e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.79769313486231570815e+308, 0xc3fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(-1.18973149535723176502e+4932, 0xfffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.67772150000000000000e+07, 0x4016, 0xffffff00L, 0x00000000L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.35544300000000000000e+07, 0x4017, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099200000e+15, 0x4034, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719871787008000000e+15, 0x4033, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(9.00719925474099100000e+15, 0x4033, 0xffffffffL, 0xfffff800L), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819840000e+16, 0x4035, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143974357401600000e+16, 0x4034, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.80143985094819820000e+16, 0x4034, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516160e+19, 0x403f, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467429741979238400e+19, 0x403e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095495680e+19, 0x403e, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.84467440737095516150e+19, 0x403e, 0xffffffffL, 0xffffffffL), ZERO_P, FLAG_IGNORE_ZEROSIGN },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032320e+19, 0x4040, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934859483958476800e+19, 0x403f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474190991360e+19, 0x403f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.68934881474191032300e+19, 0x403f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816958e+31, 0x4069, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296335789034032373e+31, 0x4068, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066726886e+31, 0x4068, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(8.11296384146066816914e+31, 0x4068, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363392e+32, 0x406a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259267157806806475e+32, 0x4069, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213345377e+32, 0x4069, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.62259276829213363383e+32, 0x4069, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552571e+34, 0x4070, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845930980996356144e+34, 0x406f, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696541041e+34, 0x406f, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.03845937170696552565e+34, 0x406f, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105141e+34, 0x4071, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691861961992712287e+34, 0x4070, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393082083e+34, 0x4070, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(2.07691874341393105130e+34, 0x4070, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.17549435082228750797e-38, 0xbf81, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.22507385850720138309e-308, 0xbc01, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-3.36210314311209350626e-4932, 0x8001, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.68105157155604675313e-4932, 0x8000, 0x40000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00416836000897277800e-292, 0xbc36, 0x80000000L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999403953712118209e-01, 0x3ffe, 0xfffff600L, 0x002d0000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+02, 0x4005, 0xc8000000L, 0x00000000L), HEXCONSTE(9.99994039553108359387e-01, 0x3ffe, 0xffff9c00L, 0x1355fd88L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+03, 0x4008, 0xfa000000L, 0x00000000L), HEXCONSTE(9.99940397129769905239e-01, 0x3ffe, 0xfffc1807L, 0x9f221888L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), HEXCONSTE(3.67879430207780343709e-01, 0x3ffd, 0xbc5ab153L, 0x3a20be44L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.07374182400000000000e+09, 0x401d, 0x80000000L, 0x00000000L), HEXCONSTE(1.60380783152492423385e-28, 0x3fa2, 0xcb4e8a2fL, 0x3c38e77bL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.22167948800000000000e+09, 0x401d, 0x91a2b300L, 0x00000000L), HEXCONSTE(2.37488471213529509998e-32, 0x3f95, 0xf69f43d1L, 0xf84c9cb7L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+01, 0xc002, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000059604664315316e+00, 0x3fff, 0x80000500L, 0x001b8000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+02, 0xc005, 0xc8000000L, 0x00000000L), HEXCONSTE(1.00000596048241877946e+00, 0x3fff, 0x80003200L, 0x09dd014fL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+03, 0xc008, 0xfa000000L, 0x00000000L), HEXCONSTE(1.00005960642294398639e+00, 0x3fff, 0x8001f403L, 0xd18efb67L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.67772160000000000000e+07, 0xc017, 0x80000000L, 0x00000000L), HEXCONSTE(2.71828190947016105406e+00, 0x4000, 0xadf854afL, 0x9ee5c6a9L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.07374182400000000000e+09, 0xc01d, 0x80000000L, 0x00000000L), HEXCONSTE(6.23516097342650579883e+27, 0x405b, 0xa12cd58dL, 0x463b2692L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.22167948800000000000e+09, 0xc01d, 0x91a2b300L, 0x00000000L), HEXCONSTE(4.21073071416963535004e+31, 0x4068, 0x84de0307L, 0x52c5106dL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), HEXCONSTE(7.38905521808664472825e+00, 0x4001, 0xec7323edL, 0xc0a63082L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(6.10839744000000000000e+08, 0x401c, 0x91a2b300L, 0x00000000L), HEXCONSTE(4.21070330065074951882e+31, 0x4068, 0x84ddca56L, 0xba2da80eL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-6.10839744000000000000e+08, 0xc01c, 0x91a2b300L, 0x00000000L), HEXCONSTE(2.37490017367277690992e-32, 0x3f95, 0xf69fad0bL, 0xc4745715L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(5.24707399690983833600e+18, 0x403d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(5.24707344715402444800e+18, 0x403d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(5.24707386985516134400e+18, 0x403d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.24707399690983833600e+18, 0x403d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.24707344715402444800e+18, 0x403d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.24707386985516134400e+18, 0x403d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.24707399690983833600e+18, 0x403d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.01186200145503886838e-253, 0x3cb6, 0xbdea0deeL, 0xfbcfce6dL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.24707344715402444800e+18, 0x403d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.01192376249517930885e-253, 0x3cb6, 0xbded059dL, 0x22e416edL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.24707386985516134400e+18, 0x403d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.01187627478272528173e-253, 0x3cb6, 0xbdeabd7fL, 0xfdaef65fL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.24707344715402444800e+18, 0xc03d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.24707399690983833600e+18, 0xc03d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.24707386985516134400e+18, 0xc03d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.24707344715402444800e+18, 0xc03d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.24707399690983833600e+18, 0xc03d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.24707386985516134400e+18, 0xc03d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.24707344715402444800e+18, 0xc03d, 0x91a2b300L, 0x00000000L), HEXCONSTE(9.88216738318529097366e+252, 0x4347, 0xac87cd32L, 0xc350b66cL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.24707399690983833600e+18, 0xc03d, 0x91a2b400L, 0x00000000L), HEXCONSTE(9.88277056122295797870e+252, 0x4347, 0xac8a7f57L, 0x5c615da0L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.24707386985516134400e+18, 0xc03d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(9.88263115680545618108e+252, 0x4347, 0xac89dfd6L, 0x7a57811bL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(2.62353699845491916800e+18, 0x403c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(2.62353672357701222400e+18, 0x403c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(2.62353693492758067200e+18, 0x403c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.62353699845491916800e+18, 0x403c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.62353672357701222400e+18, 0x403c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.62353693492758067200e+18, 0x403c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(2.62353699845491916800e+18, 0x403c, 0x91a2b400L, 0x00000000L), HEXCONSTE(9.88277056122199922381e+252, 0x4347, 0xac8a7f57L, 0x5c4ef61bL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(2.62353672357701222400e+18, 0x403c, 0x91a2b300L, 0x00000000L), HEXCONSTE(9.88216738318433227759e+252, 0x4347, 0xac87cd32L, 0xc33e4f31L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(2.62353693492758067200e+18, 0x403c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(9.88263115680449743971e+252, 0x4347, 0xac89dfd6L, 0x7a4519a7L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.62353672357701222400e+18, 0xc03c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.62353699845491916800e+18, 0xc03c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.62353693492758067200e+18, 0xc03c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.62353672357701222400e+18, 0xc03c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.62353699845491916800e+18, 0xc03c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.62353693492758067200e+18, 0xc03c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.62353672357701222400e+18, 0xc03c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.01192376249527747837e-253, 0x3cb6, 0xbded059dL, 0x22f8592bL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.62353699845491916800e+18, 0xc03c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.01186200145513703191e-253, 0x3cb6, 0xbdea0deeL, 0xfbe4105aL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.62353693492758067200e+18, 0xc03c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.01187627478282344667e-253, 0x3cb6, 0xbdeabd7fL, 0xfdc3385fL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.20764863229484099184e-4048, 0x0b78, 0x89e15c1eL, 0x4e6cded0L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.20882855270155586633e-4048, 0x0b78, 0x8a03d8c4L, 0xbcd2be3dL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.20792122264190700422e-4048, 0x0b78, 0x89e953c0L, 0x65ee9108L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.20792122264410420007e-4048, 0x0b78, 0x89e953c0L, 0x670263afL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.20792122264203681900e-4048, 0x0b78, 0x89e953c0L, 0x65fedcdaL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), HEXCONSTE(8.27247170630728032745e+4047, 0x7485, 0xed6c6574L, 0xaa8366c9L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), HEXCONSTE(8.28055423786424094193e+4047, 0x7485, 0xeda7c7faL, 0x092bd59dL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(8.27868557364220549372e+4047, 0x7485, 0xed9a0d2fL, 0x9b8fade4L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(8.27868557365726433487e+4047, 0x7485, 0xed9a0d2fL, 0x9d6ae1feL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(8.27868557365637462797e+4047, 0x7485, 0xed9a0d2fL, 0x9d4ece8aL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), HEXCONSTE(8.28055423786423466607e+4047, 0x7485, 0xeda7c7faL, 0x092ba2eaL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), HEXCONSTE(8.27247170630727405788e+4047, 0x7485, 0xed6c6574L, 0xaa833423L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(8.27868557365725806046e+4047, 0x7485, 0xed9a0d2fL, 0x9d6aaf4eL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(8.27868557364219921931e+4047, 0x7485, 0xed9a0d2fL, 0x9b8f7b34L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(8.27868557365636835356e+4047, 0x7485, 0xed9a0d2fL, 0x9d4e9bdaL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.20882855270155678246e-4048, 0x0b78, 0x8a03d8c4L, 0xbcd2dbaeL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.20764863229484190712e-4048, 0x0b78, 0x89e15c1eL, 0x4e6cfc3aL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.20792122264410511559e-4048, 0x0b78, 0x89e953c0L, 0x6702811bL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.20792122264190791975e-4048, 0x0b78, 0x89e953c0L, 0x65eeae74L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.20792122264203773453e-4048, 0x0b78, 0x89e953c0L, 0x65fefa46L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999403953712118209e-01, 0x3ffe, 0xfffff600L, 0x002d0000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+02, 0x4005, 0xc8000000L, 0x00000000L), HEXCONSTE(9.99994039553108359387e-01, 0x3ffe, 0xffff9c00L, 0x1355fd88L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+03, 0x4008, 0xfa000000L, 0x00000000L), HEXCONSTE(9.99940397129769905239e-01, 0x3ffe, 0xfffc1807L, 0x9f221888L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), HEXCONSTE(3.67879430207780343709e-01, 0x3ffd, 0xbc5ab153L, 0x3a20be44L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.07374182400000000000e+09, 0x401d, 0x80000000L, 0x00000000L), HEXCONSTE(1.60380783152492423385e-28, 0x3fa2, 0xcb4e8a2fL, 0x3c38e77bL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.22167948800000000000e+09, 0x401d, 0x91a2b300L, 0x00000000L), HEXCONSTE(2.37488471213529509998e-32, 0x3f95, 0xf69f43d1L, 0xf84c9cb7L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+01, 0xc002, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000059604664315316e+00, 0x3fff, 0x80000500L, 0x001b8000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+02, 0xc005, 0xc8000000L, 0x00000000L), HEXCONSTE(1.00000596048241877946e+00, 0x3fff, 0x80003200L, 0x09dd014fL), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+03, 0xc008, 0xfa000000L, 0x00000000L), HEXCONSTE(1.00005960642294398639e+00, 0x3fff, 0x8001f403L, 0xd18efb67L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.67772160000000000000e+07, 0xc017, 0x80000000L, 0x00000000L), HEXCONSTE(2.71828190947016105406e+00, 0x4000, 0xadf854afL, 0x9ee5c6a9L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.07374182400000000000e+09, 0xc01d, 0x80000000L, 0x00000000L), HEXCONSTE(6.23516097342650579883e+27, 0x405b, 0xa12cd58dL, 0x463b2692L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.22167948800000000000e+09, 0xc01d, 0x91a2b300L, 0x00000000L), HEXCONSTE(4.21073071416963535004e+31, 0x4068, 0x84de0307L, 0x52c5106dL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(1.67772160000000000000e+07, 0x4017, 0x80000000L, 0x00000000L), HEXCONSTE(7.38905521808664472825e+00, 0x4001, 0xec7323edL, 0xc0a63082L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(6.10839744000000000000e+08, 0x401c, 0x91a2b300L, 0x00000000L), HEXCONSTE(4.21070330065074951882e+31, 0x4068, 0x84ddca56L, 0xba2da80eL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-6.10839744000000000000e+08, 0xc01c, 0x91a2b300L, 0x00000000L), HEXCONSTE(2.37490017367277690992e-32, 0x3f95, 0xf69fad0bL, 0xc4745715L), FLAG_INEXACT },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.24707399690983833600e+18, 0x403d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.24707344715402444800e+18, 0x403d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.24707386985516134400e+18, 0x403d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(5.24707399690983833600e+18, 0x403d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(5.24707344715402444800e+18, 0x403d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(5.24707386985516134400e+18, 0x403d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.24707399690983833600e+18, 0x403d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.01186200145503886838e-253, 0x3cb6, 0xbdea0deeL, 0xfbcfce6dL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.24707344715402444800e+18, 0x403d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.01192376249517930885e-253, 0x3cb6, 0xbded059dL, 0x22e416edL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.24707386985516134400e+18, 0x403d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.01187627478272528173e-253, 0x3cb6, 0xbdeabd7fL, 0xfdaef65fL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.24707344715402444800e+18, 0xc03d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.24707399690983833600e+18, 0xc03d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.24707386985516134400e+18, 0xc03d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.24707344715402444800e+18, 0xc03d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.24707399690983833600e+18, 0xc03d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.24707386985516134400e+18, 0xc03d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.24707344715402444800e+18, 0xc03d, 0x91a2b300L, 0x00000000L), HEXCONSTE(9.88216738318529097366e+252, 0x4347, 0xac87cd32L, 0xc350b66cL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.24707399690983833600e+18, 0xc03d, 0x91a2b400L, 0x00000000L), HEXCONSTE(9.88277056122295797870e+252, 0x4347, 0xac8a7f57L, 0x5c615da0L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.24707386985516134400e+18, 0xc03d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(9.88263115680545618108e+252, 0x4347, 0xac89dfd6L, 0x7a57811bL), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.62353699845491916800e+18, 0x403c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.62353672357701222400e+18, 0x403c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(2.62353693492758067200e+18, 0x403c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(2.62353699845491916800e+18, 0x403c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(2.62353672357701222400e+18, 0x403c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(2.62353693492758067200e+18, 0x403c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(2.62353699845491916800e+18, 0x403c, 0x91a2b400L, 0x00000000L), HEXCONSTE(9.88277056122199922381e+252, 0x4347, 0xac8a7f57L, 0x5c4ef61bL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(2.62353672357701222400e+18, 0x403c, 0x91a2b300L, 0x00000000L), HEXCONSTE(9.88216738318433227759e+252, 0x4347, 0xac87cd32L, 0xc33e4f31L), FLAG_INEXACT },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(2.62353693492758067200e+18, 0x403c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(9.88263115680449743971e+252, 0x4347, 0xac89dfd6L, 0x7a4519a7L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.62353672357701222400e+18, 0xc03c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.62353699845491916800e+18, 0xc03c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.62353693492758067200e+18, 0xc03c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.62353672357701222400e+18, 0xc03c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.62353699845491916800e+18, 0xc03c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.62353693492758067200e+18, 0xc03c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.62353672357701222400e+18, 0xc03c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.01192376249527747837e-253, 0x3cb6, 0xbded059dL, 0x22f8592bL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.62353699845491916800e+18, 0xc03c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.01186200145513703191e-253, 0x3cb6, 0xbdea0deeL, 0xfbe4105aL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.62353693492758067200e+18, 0xc03c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.01187627478282344667e-253, 0x3cb6, 0xbdeabd7fL, 0xfdc3385fL), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936120730741582594e+23, 0x404c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.20764863229484099184e-4048, 0x0b78, 0x89e15c1eL, 0x4e6cded0L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936102716343073112e+23, 0x404c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.20882855270155586633e-4048, 0x0b78, 0x8a03d8c4L, 0xbcd2be3dL), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936116567413926920e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.20792122264190700422e-4048, 0x0b78, 0x89e953c0L, 0x65ee9108L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936116567413893366e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.20792122264410420007e-4048, 0x0b78, 0x89e953c0L, 0x670263afL), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.71936116567413924938e+23, 0x404c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.20792122264203681900e-4048, 0x0b78, 0x89e953c0L, 0x65fedcdaL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936102716343073112e+23, 0xc04c, 0x91a2b300L, 0x00000000L), HEXCONSTE(8.27247170630728032745e+4047, 0x7485, 0xed6c6574L, 0xaa8366c9L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936120730741582594e+23, 0xc04c, 0x91a2b400L, 0x00000000L), HEXCONSTE(8.28055423786424094193e+4047, 0x7485, 0xeda7c7faL, 0x092bd59dL), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936116567413893366e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(8.27868557364220549372e+4047, 0x7485, 0xed9a0d2fL, 0x9b8fade4L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936116567413926920e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(8.27868557365726433487e+4047, 0x7485, 0xed9a0d2fL, 0x9d6ae1feL), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.71936116567413924938e+23, 0xc04c, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(8.27868557365637462797e+4047, 0x7485, 0xed9a0d2fL, 0x9d4ece8aL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680603653707912970e+22, 0x404b, 0x91a2b400L, 0x00000000L), HEXCONSTE(8.28055423786423466607e+4047, 0x7485, 0xeda7c7faL, 0x092ba2eaL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680513581715365560e+22, 0x404b, 0x91a2b300L, 0x00000000L), HEXCONSTE(8.27247170630727405788e+4047, 0x7485, 0xed6c6574L, 0xaa833423L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680582837069634601e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(8.27868557365725806046e+4047, 0x7485, 0xed9a0d2fL, 0x9d6aaf4eL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680582837069466829e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(8.27868557364219921931e+4047, 0x7485, 0xed9a0d2fL, 0x9b8f7b34L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(8.59680582837069624689e+22, 0x404b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(8.27868557365636835356e+4047, 0x7485, 0xed9a0d2fL, 0x9d4e9bdaL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680513581715365560e+22, 0xc04b, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.20882855270155678246e-4048, 0x0b78, 0x8a03d8c4L, 0xbcd2dbaeL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680603653707912970e+22, 0xc04b, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.20764863229484190712e-4048, 0x0b78, 0x89e15c1eL, 0x4e6cfc3aL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680582837069466829e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.20792122264410511559e-4048, 0x0b78, 0x89e953c0L, 0x6702811bL), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680582837069634601e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.20792122264190791975e-4048, 0x0b78, 0x89e953c0L, 0x65eeae74L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-8.59680582837069624689e+22, 0xc04b, 0x91a2b3c4L, 0xd5e6f787L), HEXCONSTE(1.20792122264203773453e-4048, 0x0b78, 0x89e953c0L, 0x65fefa46L), 0 },
#endif
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914311568120504738e+37, 0x407d, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914210156072486479e+37, 0x407d, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669406345e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669217451e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669394631e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(9.67914288130669394539e+37, 0x407d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999940395355224609e-01, 0xbffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999888978e-01, 0xbffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914210156072486479e+37, 0xc07d, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914311568120504738e+37, 0xc07d, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669217451e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669406345e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669394539e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-9.99999999999999999946e-01, 0xbffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-9.67914288130669394631e+37, 0xc07d, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957155784060252369e+37, 0x407c, 0x91a2b400L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957105078036243240e+37, 0x407c, 0x91a2b300L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334703173e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f800L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334608725e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334697316e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f781L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(4.83957144065334697270e+37, 0x407c, 0x91a2b3c4L, 0xd5e6f780L), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000022204e+00, 0xbfff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957105078036243240e+37, 0xc07c, 0x91a2b300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957155784060252369e+37, 0xc07c, 0x91a2b400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334608725e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334703173e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334697270e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f780L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000000000000000011e+00, 0xbfff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.83957144065334697316e+37, 0xc07c, 0x91a2b3c4L, 0xd5e6f781L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.07374182400000000000e+09, 0x401d, 0x80000000L, 0x00000000L), HEXCONSTE(3.88767874524884743622e+55, 0x40b7, 0xcaf240b5L, 0xb0dee3a8L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(1.07374182400000000000e+09, 0x401d, 0x80000000L, 0x00000000L), HEXCONSTE(3.88767874524884743622e+55, 0x40b7, 0xcaf240b5L, 0xb0dee3a8L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000011920928955078e+00, 0xbfff, 0x80000100L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000083446502685547e+00, 0x3fff, 0x80000700L, 0x00000000L), HEXCONSTE(1.07374182400000000000e+09, 0x401d, 0x80000000L, 0x00000000L), HEXCONSTE(1.34181742881446768735e+389, 0x450b, 0xc97107c6L, 0xe5829f58L), 0 },
	{ __LINE__,  HEXCONSTE(-1.00000083446502685547e+00, 0xbfff, 0x80000700L, 0x00000000L), HEXCONSTE(1.07374182400000000000e+09, 0x401d, 0x80000000L, 0x00000000L), HEXCONSTE(1.34181742881446768735e+389, 0x450b, 0xc97107c6L, 0xe5829f58L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000083446502685547e+00, 0x3fff, 0x80000700L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(1.00000083446502685547e+00, 0x3fff, 0x80000700L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000083446502685547e+00, 0x3fff, 0x80000700L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(-1.00000083446502685547e+00, 0xbfff, 0x80000700L, 0x00000000L), HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), INF_P, 0 },
	{ __LINE__,  HEXCONSTE(-1.00000083446502685547e+00, 0xbfff, 0x80000700L, 0x00000000L), HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), INF_P, 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(-1.00000083446502685547e+00, 0xbfff, 0x80000700L, 0x00000000L), HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), INF_P, 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(7.92281589724894490530e+28, 0x405e, 0xffffff3fL, 0xffffe800L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.55251809230070880588e+231, 0x42fe, 0xffffffffL, 0xfffffa00L), FLAG_INEXACT },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000001e+4932, 0x7ffe, 0xd72cb2a9L, 0x5c7ef6cdL), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e+3699, 0x6ffe, 0xe0b9d894L, 0xb9a782d8L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999942e+4931, 0x7ffe, 0xd72cb2a9L, 0x5c7ef6ccL), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999944e+3698, 0x6ffe, 0xe0b9d894L, 0xb9a782d7L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(7.92281589724894490530e+28, 0x405e, 0xffffff3fL, 0xffffe800L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.55251809230070880588e+231, 0x42fe, 0xffffffffL, 0xfffffa00L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000005e+4928, 0x7ff1, 0xb045626fL, 0xb50a35e8L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000007e+3696, 0x6ff4, 0xe61e8ff4L, 0x7461cdabL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999973e+4927, 0x7ff1, 0xb045626fL, 0xb50a35e7L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e+3696, 0x6ff4, 0xe61e8ff4L, 0x7461cdaaL), 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(7.92281589724894490530e+28, 0x405e, 0xffffff3fL, 0xffffe800L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.55251809230070880588e+231, 0x42fe, 0xffffffffL, 0xfffffa00L), FLAG_INEXACT },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000006e+4924, 0x7fe4, 0x9066b70aL, 0x00085d50L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000004e+3693, 0x6fea, 0xeba46a73L, 0x274d9f67L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999968e+4923, 0x7fe4, 0x9066b70aL, 0x00085d4fL), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999984e+3692, 0x6fea, 0xeba46a73L, 0x274d9f66L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(7.92281589724894490530e+28, 0x405e, 0xffffff3fL, 0xffffe800L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.55251809230070880588e+231, 0x42fe, 0xffffffffL, 0xfffffa00L), FLAG_INEXACT },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00000000000000000004e+4920, 0x7fd6, 0xec96440eL, 0xbeed5893L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000003e+3690, 0x6fe0, 0xf14c33a9L, 0x1e01a95fL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999978e+4919, 0x7fd6, 0xec96440eL, 0xbeed5892L), HEXCONSTE(7.50000000000000000000e-01, 0x3ffe, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999972e+3689, 0x6fe0, 0xf14c33a9L, 0x1e01a95eL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(4.93200000000000000000e+03, 0x400b, 0x9a200000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e+4932, 0x7ffe, 0xd72cb2a9L, 0x5c7ef6cdL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(4.93100000000000000000e+03, 0x400b, 0x9a180000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e+4931, 0x7ffb, 0xac23c221L, 0x16cbf8a4L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(4.93000000000000000000e+03, 0x400b, 0x9a100000L, 0x00000000L), HEXCONSTE(1.00000000000000000003e+4930, 0x7ff8, 0x89b634e7L, 0x456ffa1dL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(4.92900000000000000000e+03, 0x400b, 0x9a080000L, 0x00000000L), HEXCONSTE(9.99999999999999999988e+4928, 0x7ff4, 0xdc56bb0bL, 0xa24cc361L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.93100000000000000000e+03, 0xc00b, 0x9a180000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e-4931, 0x0002, 0xbe5b66ecL, 0xbce0b7b1L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.93000000000000000000e+03, 0xc00b, 0x9a100000L, 0x00000000L), HEXCONSTE(9.99999999999999999994e-4931, 0x0005, 0xedf240a7L, 0xec18e59dL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.92900000000000000000e+03, 0xc00b, 0x9a080000L, 0x00000000L), HEXCONSTE(9.99999999999999999982e-4930, 0x0009, 0x94b76868L, 0xf38f8f82L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000006227113104857e+27, 0x4058, 0xcecb9000L, 0x00000000L), HEXCONSTE(1.82000000000000000000e+02, 0x4006, 0xb6000000L, 0x00000000L), HEXCONSTE(1.00001133340972057289e+4914, 0x7fc2, 0xf8150d73L, 0xa5bad7b7L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999988484154753735e+26, 0x4058, 0xcecb8f00L, 0x00000000L), HEXCONSTE(1.82000000000000000000e+02, 0x4006, 0xb6000000L, 0x00000000L), HEXCONSTE(9.99997904118349474843e+4913, 0x7fc2, 0xf814331dL, 0xd58d7f20L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000001329e+27, 0x4058, 0xcecb8f27L, 0xf4201000L), HEXCONSTE(1.82000000000000000000e+02, 0x4006, 0xb6000000L, 0x00000000L), HEXCONSTE(1.00000000000000241834e+4914, 0x7fc2, 0xf8145531L, 0x10d16d4aL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999875849e+26, 0x4058, 0xcecb8f27L, 0xf4200800L), HEXCONSTE(1.82000000000000000000e+02, 0x4006, 0xb6000000L, 0x00000000L), HEXCONSTE(9.99999999999977404472e+4913, 0x7fc2, 0xf8145531L, 0x10ca9a9eL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+27, 0x4058, 0xcecb8f27L, 0xf4200f3aL), HEXCONSTE(1.82000000000000000000e+02, 0x4006, 0xb6000000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e+4914, 0x7fc2, 0xf8145531L, 0x10d0c46cL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000006227113104857e+27, 0x4058, 0xcecb9000L, 0x00000000L), HEXCONSTE(-1.82000000000000000000e+02, 0xc006, 0xb6000000L, 0x00000000L), HEXCONSTE(9.99988666718724147310e-4915, 0x003b, 0x8415d1d3L, 0x9fa9bc66L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999988484154753735e+26, 0x4058, 0xcecb8f00L, 0x00000000L), HEXCONSTE(-1.82000000000000000000e+02, 0xc006, 0xb6000000L, 0x00000000L), HEXCONSTE(1.00000209588604325430e-4914, 0x003b, 0x84164613L, 0x4c345c43L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000001329e+27, 0x4058, 0xcecb8f27L, 0xf4201000L), HEXCONSTE(-1.82000000000000000000e+02, 0xc006, 0xb6000000L, 0x00000000L), HEXCONSTE(9.99999999999997581630e-4915, 0x003b, 0x841633eeL, 0xb6e53abfL), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999875849e+26, 0x4058, 0xcecb8f27L, 0xf4200800L), HEXCONSTE(-1.82000000000000000000e+02, 0xc006, 0xb6000000L, 0x00000000L), HEXCONSTE(1.00000000000002259557e-4914, 0x003b, 0x841633eeL, 0xb6e8dcbeL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+27, 0x4058, 0xcecb8f27L, 0xf4200f3aL), HEXCONSTE(-1.82000000000000000000e+02, 0xc006, 0xb6000000L, 0x00000000L), HEXCONSTE(1.00000000000000000003e-4914, 0x003b, 0x841633eeL, 0xb6e594a9L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.79769313486231570815e+308, 0x43fe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.18973149535723176502e+4932, 0x7ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.90000009536743164062e-01, 0x3ffe, 0xfd70a400L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.89999949932098388672e-01, 0x3ffe, 0xfd70a300L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.90000000000000102141e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7800L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.89999999999999991118e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d7000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.90000000000000000009e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a4L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(9.89999999999999999954e-01, 0x3ffe, 0xfd70a3d7L, 0x0a3d70a3L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(1.40129846432481707092e-45, 0x3f6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(4.94065645841246544177e-324, 0x3bcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(3.64519953188247460253e-4951, 0x0000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.01000010967254638672e+00, 0x3fff, 0x8147af00L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00999999046325683594e+00, 0x3fff, 0x8147ae00L, 0x00000000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.01000000000000000888e+00, 0x3fff, 0x8147ae14L, 0x7ae14800L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00999999999999978684e+00, 0x3fff, 0x8147ae14L, 0x7ae14000L), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.01000000000000000010e+00, 0x3fff, 0x8147ae14L, 0x7ae147afL), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(-1.40129846432481707092e-45, 0xbf6a, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(-4.94065645841246544177e-324, 0xbbcd, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(-3.64519953188247460253e-4951, 0x8000, 0x00000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00999999999999999999e+00, 0x3fff, 0x8147ae14L, 0x7ae147aeL), HEXCONSTE(-0.0, 0x8000, 0x00000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+05, 0xc00f, 0xc3500000L, 0x00000000L), HEXCONSTE(0.00000000000000000000e+00, 0x0000, 0x00000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26125000000000000000e+02, 0xc005, 0xfc400000L, 0x00000000L), HEXCONSTE(1.07793307246828789602e-38, 0x3f80, 0xeac0c6e7L, 0xdd24392fL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26250000000000000000e+02, 0xc005, 0xfc800000L, 0x00000000L), HEXCONSTE(9.88468985757453879590e-39, 0x3f80, 0xd744fccaL, 0xd69d6af4L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26375000000000000000e+02, 0xc005, 0xfcc00000L, 0x00000000L), HEXCONSTE(9.06430056522005789823e-39, 0x3f80, 0xc5672a11L, 0x5506daddL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26500000000000000000e+02, 0xc005, 0xfd000000L, 0x00000000L), HEXCONSTE(8.31200026712917983887e-39, 0x3f80, 0xb504f333L, 0xf9de6484L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26625000000000000000e+02, 0xc005, 0xfd400000L, 0x00000000L), HEXCONSTE(7.62213785207576524858e-39, 0x3f80, 0xa5fed6a9L, 0xb15138eaL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26750000000000000000e+02, 0xc005, 0xfd800000L, 0x00000000L), HEXCONSTE(6.98953122821684501786e-39, 0x3f80, 0x9837f051L, 0x8db8a96fL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.26875000000000000000e+02, 0xc005, 0xfdc00000L, 0x00000000L), HEXCONSTE(6.40942839638015852804e-39, 0x3f80, 0x8b95c1e3L, 0xea8bd6e7L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69125000000000000000e+02, 0xc008, 0xf2480000L, 0x00000000L), HEXCONSTE(1.83783048939110316058e-292, 0x3c35, 0xeac0c6e7L, 0xdd24392fL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69250000000000000000e+02, 0xc008, 0xf2500000L, 0x00000000L), HEXCONSTE(1.68529798949646123644e-292, 0x3c35, 0xd744fccaL, 0xd69d6af4L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69375000000000000000e+02, 0xc008, 0xf2580000L, 0x00000000L), HEXCONSTE(1.54542507037295850337e-292, 0x3c35, 0xc5672a11L, 0x5506daddL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69500000000000000000e+02, 0xc008, 0xf2600000L, 0x00000000L), HEXCONSTE(1.41716103800186651817e-292, 0x3c35, 0xb504f333L, 0xf9de6484L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69625000000000000000e+02, 0xc008, 0xf2680000L, 0x00000000L), HEXCONSTE(1.29954240171984033396e-292, 0x3c35, 0xa5fed6a9L, 0xb15138eaL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69750000000000000000e+02, 0xc008, 0xf2700000L, 0x00000000L), HEXCONSTE(1.19168563669300268360e-292, 0x3c35, 0x9837f051L, 0x8db8a96fL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-9.69875000000000000000e+02, 0xc008, 0xf2780000L, 0x00000000L), HEXCONSTE(1.09278054707641637781e-292, 0x3c35, 0x8b95c1e3L, 0xea8bd6e7L), FLAG_INEXACT },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02212500000000000000e+03, 0xc008, 0xff880000L, 0x00000000L), HEXCONSTE(2.04040172468012222027e-308, 0x3c00, 0xeac0c6e7L, 0xdd24392fL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02225000000000000000e+03, 0xc008, 0xff900000L, 0x00000000L), HEXCONSTE(1.87105663129345649192e-308, 0x3c00, 0xd744fccaL, 0xd69d6af4L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02237500000000000000e+03, 0xc008, 0xff980000L, 0x00000000L), HEXCONSTE(1.71576649596101138792e-308, 0x3c00, 0xc5672a11L, 0x5506daddL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02250000000000000000e+03, 0xc008, 0xffa00000L, 0x00000000L), HEXCONSTE(1.57336481399135865519e-308, 0x3c00, 0xb504f333L, 0xf9de6484L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02262500000000000000e+03, 0xc008, 0xffa80000L, 0x00000000L), HEXCONSTE(1.44278189586604136836e-308, 0x3c00, 0xa5fed6a9L, 0xb15138eaL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02275000000000000000e+03, 0xc008, 0xffb00000L, 0x00000000L), HEXCONSTE(1.32303683197166086893e-308, 0x3c00, 0x9837f051L, 0x8db8a96fL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02287500000000000000e+03, 0xc008, 0xffb80000L, 0x00000000L), HEXCONSTE(1.21323012422671225631e-308, 0x3c00, 0x8b95c1e3L, 0xea8bd6e7L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63821250000000000000e+04, 0xc00c, 0xfff88000L, 0x00000000L), HEXCONSTE(3.08306217590492313920e-4932, 0x0000, 0x75606373L, 0xee921c98L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63822500000000000000e+04, 0xc00c, 0xfff90000L, 0x00000000L), HEXCONSTE(2.82718048075620583510e-4932, 0x0000, 0x6ba27e65L, 0x6b4eb57aL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63823750000000000000e+04, 0xc00c, 0xfff98000L, 0x00000000L), HEXCONSTE(2.59253593172276695929e-4932, 0x0000, 0x62b39508L, 0xaa836d6eL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63825000000000000000e+04, 0xc00c, 0xfffa0000L, 0x00000000L), HEXCONSTE(2.37736593154316677945e-4932, 0x0000, 0x5a827999L, 0xfcef3242L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63826250000000000000e+04, 0xc00c, 0xfffa8000L, 0x00000000L), HEXCONSTE(2.18005417140212357933e-4932, 0x0000, 0x52ff6b54L, 0xd8a89c75L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63827500000000000000e+04, 0xc00c, 0xfffb0000L, 0x00000000L), HEXCONSTE(1.99911848958095667371e-4932, 0x0000, 0x4c1bf828L, 0xc6dc54b8L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63828750000000000000e+04, 0xc00c, 0xfffb8000L, 0x00000000L), HEXCONSTE(1.83319973779095268933e-4932, 0x0000, 0x45cae0f1L, 0xf545eb74L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63831250000000000000e+04, 0xc00c, 0xfffc8000L, 0x00000000L), HEXCONSTE(1.54153108795246156960e-4932, 0x0000, 0x3ab031b9L, 0xf7490e4cL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63832500000000000000e+04, 0xc00c, 0xfffd0000L, 0x00000000L), HEXCONSTE(1.41359024037810291755e-4932, 0x0000, 0x35d13f32L, 0xb5a75abdL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63833750000000000000e+04, 0xc00c, 0xfffd8000L, 0x00000000L), HEXCONSTE(1.29626796586138348001e-4932, 0x0000, 0x3159ca84L, 0x5541b6b8L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63835000000000000000e+04, 0xc00c, 0xfffe0000L, 0x00000000L), HEXCONSTE(1.18868296577158338972e-4932, 0x0000, 0x2d413cccL, 0xfe779921L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63836250000000000000e+04, 0xc00c, 0xfffe8000L, 0x00000000L), HEXCONSTE(1.09002708570106178948e-4932, 0x0000, 0x297fb5aaL, 0x6c544e3aL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63837500000000000000e+04, 0xc00c, 0xffff0000L, 0x00000000L), HEXCONSTE(9.99559244790478336855e-4933, 0x0000, 0x260dfc14L, 0x636e2a5cL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.63838750000000000000e+04, 0xc00c, 0xffff8000L, 0x00000000L), HEXCONSTE(9.16599868895476344664e-4933, 0x0000, 0x22e57078L, 0xfaa2f5baL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26125000000000000000e+02, 0x4005, 0xfc400000L, 0x00000000L), HEXCONSTE(1.07793307246828789602e-38, 0x3f80, 0xeac0c6e7L, 0xdd24392fL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26250000000000000000e+02, 0x4005, 0xfc800000L, 0x00000000L), HEXCONSTE(9.88468985757453879590e-39, 0x3f80, 0xd744fccaL, 0xd69d6af4L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26375000000000000000e+02, 0x4005, 0xfcc00000L, 0x00000000L), HEXCONSTE(9.06430056522005789823e-39, 0x3f80, 0xc5672a11L, 0x5506daddL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26500000000000000000e+02, 0x4005, 0xfd000000L, 0x00000000L), HEXCONSTE(8.31200026712917983887e-39, 0x3f80, 0xb504f333L, 0xf9de6484L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26625000000000000000e+02, 0x4005, 0xfd400000L, 0x00000000L), HEXCONSTE(7.62213785207576524858e-39, 0x3f80, 0xa5fed6a9L, 0xb15138eaL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26750000000000000000e+02, 0x4005, 0xfd800000L, 0x00000000L), HEXCONSTE(6.98953122821684501786e-39, 0x3f80, 0x9837f051L, 0x8db8a96fL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.26875000000000000000e+02, 0x4005, 0xfdc00000L, 0x00000000L), HEXCONSTE(6.40942839638015852804e-39, 0x3f80, 0x8b95c1e3L, 0xea8bd6e7L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69125000000000000000e+02, 0x4008, 0xf2480000L, 0x00000000L), HEXCONSTE(1.83783048939110316058e-292, 0x3c35, 0xeac0c6e7L, 0xdd24392fL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69250000000000000000e+02, 0x4008, 0xf2500000L, 0x00000000L), HEXCONSTE(1.68529798949646123644e-292, 0x3c35, 0xd744fccaL, 0xd69d6af4L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69375000000000000000e+02, 0x4008, 0xf2580000L, 0x00000000L), HEXCONSTE(1.54542507037295850337e-292, 0x3c35, 0xc5672a11L, 0x5506daddL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69500000000000000000e+02, 0x4008, 0xf2600000L, 0x00000000L), HEXCONSTE(1.41716103800186651817e-292, 0x3c35, 0xb504f333L, 0xf9de6484L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69625000000000000000e+02, 0x4008, 0xf2680000L, 0x00000000L), HEXCONSTE(1.29954240171984033396e-292, 0x3c35, 0xa5fed6a9L, 0xb15138eaL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69750000000000000000e+02, 0x4008, 0xf2700000L, 0x00000000L), HEXCONSTE(1.19168563669300268360e-292, 0x3c35, 0x9837f051L, 0x8db8a96fL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(9.69875000000000000000e+02, 0x4008, 0xf2780000L, 0x00000000L), HEXCONSTE(1.09278054707641637781e-292, 0x3c35, 0x8b95c1e3L, 0xea8bd6e7L), FLAG_INEXACT },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02212500000000000000e+03, 0x4008, 0xff880000L, 0x00000000L), HEXCONSTE(2.04040172468012222027e-308, 0x3c00, 0xeac0c6e7L, 0xdd24392fL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02225000000000000000e+03, 0x4008, 0xff900000L, 0x00000000L), HEXCONSTE(1.87105663129345649192e-308, 0x3c00, 0xd744fccaL, 0xd69d6af4L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02237500000000000000e+03, 0x4008, 0xff980000L, 0x00000000L), HEXCONSTE(1.71576649596101138792e-308, 0x3c00, 0xc5672a11L, 0x5506daddL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02250000000000000000e+03, 0x4008, 0xffa00000L, 0x00000000L), HEXCONSTE(1.57336481399135865519e-308, 0x3c00, 0xb504f333L, 0xf9de6484L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02262500000000000000e+03, 0x4008, 0xffa80000L, 0x00000000L), HEXCONSTE(1.44278189586604136836e-308, 0x3c00, 0xa5fed6a9L, 0xb15138eaL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02275000000000000000e+03, 0x4008, 0xffb00000L, 0x00000000L), HEXCONSTE(1.32303683197166086893e-308, 0x3c00, 0x9837f051L, 0x8db8a96fL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.02287500000000000000e+03, 0x4008, 0xffb80000L, 0x00000000L), HEXCONSTE(1.21323012422671225631e-308, 0x3c00, 0x8b95c1e3L, 0xea8bd6e7L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63821250000000000000e+04, 0x400c, 0xfff88000L, 0x00000000L), HEXCONSTE(3.08306217590492313920e-4932, 0x0000, 0x75606373L, 0xee921c98L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63822500000000000000e+04, 0x400c, 0xfff90000L, 0x00000000L), HEXCONSTE(2.82718048075620583510e-4932, 0x0000, 0x6ba27e65L, 0x6b4eb57aL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63823750000000000000e+04, 0x400c, 0xfff98000L, 0x00000000L), HEXCONSTE(2.59253593172276695929e-4932, 0x0000, 0x62b39508L, 0xaa836d6eL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63825000000000000000e+04, 0x400c, 0xfffa0000L, 0x00000000L), HEXCONSTE(2.37736593154316677945e-4932, 0x0000, 0x5a827999L, 0xfcef3242L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63826250000000000000e+04, 0x400c, 0xfffa8000L, 0x00000000L), HEXCONSTE(2.18005417140212357933e-4932, 0x0000, 0x52ff6b54L, 0xd8a89c75L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63827500000000000000e+04, 0x400c, 0xfffb0000L, 0x00000000L), HEXCONSTE(1.99911848958095667371e-4932, 0x0000, 0x4c1bf828L, 0xc6dc54b8L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63828750000000000000e+04, 0x400c, 0xfffb8000L, 0x00000000L), HEXCONSTE(1.83319973779095268933e-4932, 0x0000, 0x45cae0f1L, 0xf545eb74L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63831250000000000000e+04, 0x400c, 0xfffc8000L, 0x00000000L), HEXCONSTE(1.54153108795246156960e-4932, 0x0000, 0x3ab031b9L, 0xf7490e4cL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63832500000000000000e+04, 0x400c, 0xfffd0000L, 0x00000000L), HEXCONSTE(1.41359024037810291755e-4932, 0x0000, 0x35d13f32L, 0xb5a75abdL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63833750000000000000e+04, 0x400c, 0xfffd8000L, 0x00000000L), HEXCONSTE(1.29626796586138348001e-4932, 0x0000, 0x3159ca84L, 0x5541b6b8L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63835000000000000000e+04, 0x400c, 0xfffe0000L, 0x00000000L), HEXCONSTE(1.18868296577158338972e-4932, 0x0000, 0x2d413cccL, 0xfe779921L), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63836250000000000000e+04, 0x400c, 0xfffe8000L, 0x00000000L), HEXCONSTE(1.09002708570106178948e-4932, 0x0000, 0x297fb5aaL, 0x6c544e3aL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63837500000000000000e+04, 0x400c, 0xffff0000L, 0x00000000L), HEXCONSTE(9.99559244790478336855e-4933, 0x0000, 0x260dfc14L, 0x636e2a5cL), 0 },
	{ __LINE__,  HEXCONSTE(5.00000000000000000000e-01, 0x3ffe, 0x80000000L, 0x00000000L), HEXCONSTE(1.63838750000000000000e+04, 0x400c, 0xffff8000L, 0x00000000L), HEXCONSTE(9.16599868895476344664e-4933, 0x0000, 0x22e57078L, 0xfaa2f5baL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(2.00000190734863281250e+00, 0x4000, 0x80000800L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17535310849065169555e-38, 0x3f80, 0xfff8201fL, 0x40aca8a8L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000381469726562500e+00, 0xc000, 0x80001000L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17521188326476003093e-38, 0x3f80, 0xfff0407dL, 0x01654a80L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000572204589843750e+00, 0x4000, 0x80001800L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17507067514252454093e-38, 0x3f80, 0xffe86119L, 0x4035ed26L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-2.00000762939453125000e+00, 0xc000, 0x80002000L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17492948412185750939e-38, 0x3f80, 0xffe081f3L, 0xfb2aa7f9L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000953674316406250e+00, 0x4000, 0x80002800L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17478831020067147688e-38, 0x3f80, 0xffd8a30dL, 0x304fa215L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00001144409179687500e+00, 0xc000, 0x80003000L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17464715337687924064e-38, 0x3f80, 0xffd0c464L, 0xddb11252L), 0 },
	{ __LINE__,  HEXCONSTE(2.00001335144042968750e+00, 0x4000, 0x80003800L, 0x00000000L), HEXCONSTE(-1.26000000000000000000e+02, 0xc005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17450601364839385492e-38, 0x3f80, 0xffc8e5fbL, 0x015b3f49L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000190734863281250e+00, 0x4000, 0x80000800L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(2.00231714353756874151e-292, 0x3c35, 0xffc3772bL, 0x3bf7e275L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000381469726562500e+00, 0xc000, 0x80001000L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(-2.00046763876656887574e-292, 0xbc35, 0xff86fcaaL, 0xac03e34aL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000572204589843750e+00, 0x4000, 0x80001800L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.99861984411164414236e-292, 0x3c35, 0xff4a907aL, 0xeaf545c9L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-2.00000762939453125000e+00, 0xc000, 0x80002000L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(-1.99677375798993344742e-292, 0xbc35, 0xff0e3298L, 0x946b7781L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000953674316406250e+00, 0x4000, 0x80002800L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.99492937882004227995e-292, 0x3c35, 0xfed1e300L, 0x44d3df54L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-2.00001144409179687500e+00, 0xc000, 0x80003000L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(-1.99308670502204135197e-292, 0xbc35, 0xfe95a1aeL, 0x9969ac91L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00001335144042968750e+00, 0x4000, 0x80003800L, 0x00000000L), HEXCONSTE(-9.69000000000000000000e+02, 0xc008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.99124573501746523888e-292, 0x3c35, 0xfe596ea0L, 0x3035a612L), FLAG_INEXACT },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(2.00000190734863281250e+00, 0x4000, 0x80000800L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.22290623635608018625e-308, 0x3c00, 0xffc027f9L, 0x56dfe4ccL), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000381469726562500e+00, 0xc000, 0x80001000L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.22074072792299609502e-308, 0x3c00, 0xff805fe2L, 0xb354443bL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000572204589843750e+00, 0x4000, 0x80001800L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.21857733114478185967e-308, 0x3c00, 0xff40a7b8L, 0x19db8e41L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000762939453125000e+00, 0xc000, 0x80002000L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.21641604396028602061e-308, 0x3c00, 0xff00ff75L, 0x8ff332e5L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000953674316406250e+00, 0x4000, 0x80002800L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.21425686431037093693e-308, 0x3c00, 0xfec16717L, 0x1c176269L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00001144409179687500e+00, 0xc000, 0x80003000L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.21209979013791081705e-308, 0x3c00, 0xfe81de98L, 0xc5c2cd83L), 0 },
	{ __LINE__,  HEXCONSTE(2.00001335144042968750e+00, 0x4000, 0x80003800L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.20994481938778975083e-308, 0x3c00, 0xfe4265f6L, 0x956e65a3L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000190734863281250e+00, 0x4000, 0x80000800L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.30998490318593574615e-4932, 0x0000, 0x7e040a80L, 0x3c133cc2L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000381469726562500e+00, 0xc000, 0x80001000L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.25867463197883697768e-4932, 0x0000, 0x7c0ff4ecL, 0x2227618cL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000572204589843750e+00, 0x4000, 0x80001800L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.20815980311461223662e-4932, 0x0000, 0x7a23a002L, 0xb22d44bcL), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000762939453125000e+00, 0xc000, 0x80002000L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.15842808443193287565e-4932, 0x0000, 0x783eecfeL, 0xf93700c0L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000953674316406250e+00, 0x4000, 0x80002800L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.10946733497294672390e-4932, 0x0000, 0x7661bd96L, 0x250fba1cL), 0 },
	{ __LINE__,  HEXCONSTE(-2.00001144409179687500e+00, 0xc000, 0x80003000L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.06126560201859370891e-4932, 0x0000, 0x748bf3f5L, 0x9f7610f4L), 0 },
	{ __LINE__,  HEXCONSTE(2.00001335144042968750e+00, 0x4000, 0x80003800L, 0x00000000L), HEXCONSTE(-1.63820000000000000000e+04, 0xc00c, 0xfff80000L, 0x00000000L), HEXCONSTE(3.01381111816989283009e-4932, 0x0000, 0x72bd72c1L, 0x30daee6cL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000190734863281250e+00, 0x4000, 0x80000800L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.65499087327067814818e-4932, 0x0000, 0x3f02014fL, 0xfdf49e82L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000381469726562500e+00, 0xc000, 0x80001000L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(-1.62933420828104392922e-4932, 0x8000, 0x3e07f2b5L, 0x12bd0e6eL), 0 },
	{ __LINE__,  HEXCONSTE(2.00000572204589843750e+00, 0x4000, 0x80001800L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.60407531226102546401e-4932, 0x0000, 0x3d11c48eL, 0x043c0192L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00000762939453125000e+00, 0xc000, 0x80002000L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(-1.57920801801545826132e-4932, 0x8000, 0x3c1f6777L, 0xa2bd97b1L), 0 },
	{ __LINE__,  HEXCONSTE(2.00000953674316406250e+00, 0x4000, 0x80002800L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.55472625397398607488e-4932, 0x0000, 0x3b30cc4bL, 0xd2b02b38L), 0 },
	{ __LINE__,  HEXCONSTE(-2.00001144409179687500e+00, 0xc000, 0x80003000L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(-1.53062404270827122597e-4932, 0x8000, 0x3a45e420L, 0x9a2ecea9L), 0 },
	{ __LINE__,  HEXCONSTE(2.00001335144042968750e+00, 0x4000, 0x80003800L, 0x00000000L), HEXCONSTE(-1.63830000000000000000e+04, 0xc00c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.50689549947219893151e-4932, 0x0000, 0x395ea047L, 0x324e5134L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(4.99992847442626953125e-01, 0x3ffd, 0xffff1000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17337748074663812902e-38, 0x3f80, 0xff89fb05L, 0x4bca3399L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99993324279785156250e-01, 0xbffd, 0xffff2000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17351848768407883598e-38, 0x3f80, 0xff91d789L, 0xf88ded26L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(4.99993801116943359375e-01, 0x3ffd, 0xffff3000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17365951143208278985e-38, 0x3f80, 0xff99b44cL, 0x1143d750L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-4.99994277954101562500e-01, 0xbffd, 0xffff4000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17380055199263807726e-38, 0x3f80, 0xffa1914bL, 0x97c7ff72L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(4.99994754791259765625e-01, 0x3ffd, 0xffff5000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17394160936773301798e-38, 0x3f80, 0xffa96e88L, 0x8df68132L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99995231628417968750e-01, 0xbffd, 0xffff6000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17408268355935616518e-38, 0x3f80, 0xffb14c02L, 0xf5ab8684L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(4.99995708465576171875e-01, 0x3ffd, 0xffff7000L, 0x00000000L), HEXCONSTE(1.26000000000000000000e+02, 0x4005, 0xfc000000L, 0x00000000L), HEXCONSTE(1.17422377456949630528e-38, 0x3f80, 0xffb929baL, 0xd0c347a9L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(4.99992847442626953125e-01, 0x3ffd, 0xffff1000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.97657873066936000348e-292, 0x3c35, 0xfc79d2c5L, 0x6315dbb2L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-4.99993324279785156250e-01, 0xbffd, 0xffff2000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(-1.97840617717724951142e-292, 0xbc35, 0xfcb5949eL, 0xb638d153L), 0 },
	{ __LINE__,  HEXCONSTE(4.99993801116943359375e-01, 0x3ffd, 0xffff3000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.98023531150617315725e-292, 0x3c35, 0xfcf16499L, 0x10eb1ddfL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(-4.99994277954101562500e-01, 0xbffd, 0xffff4000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(-1.98206613521338265959e-292, 0xbc35, 0xfd2d42b7L, 0xc9809fd0L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(4.99994754791259765625e-01, 0x3ffd, 0xffff5000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.98389864985756503325e-292, 0x3c35, 0xfd692efeL, 0x3716ca11L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99995231628417968750e-01, 0xbffd, 0xffff6000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(-1.98573285699884391032e-292, 0xbc35, 0xfda5296fL, 0xb194d37eL), 0 },
#if defined TEST_LDOUBLE && LDBL_MANT_DIG >= 64
	{ __LINE__,  HEXCONSTE(4.99995708465576171875e-01, 0x3ffd, 0xffff7000L, 0x00000000L), HEXCONSTE(9.69000000000000000000e+02, 0x4008, 0xf2400000L, 0x00000000L), HEXCONSTE(1.98756875819878086331e-292, 0x3c35, 0xfde1320fL, 0x91abe676L), 0 },
	{ __LINE__,  HEXCONSTE(4.99992847442626953125e-01, 0x3ffd, 0xffff1000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.19278007204446471136e-308, 0x3c00, 0xfc48d68aL, 0x75dd1bc7L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99993324279785156250e-01, 0xbffd, 0xffff2000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.19491834787767397184e-308, 0x3c00, 0xfc87d14bL, 0x25ac6f34L), 0 },
	{ __LINE__,  HEXCONSTE(4.99993801116943359375e-01, 0x3ffd, 0xffff3000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.19705870679501885051e-308, 0x3c00, 0xfcc6dbc0L, 0xb9708f15L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99994277954101562500e-01, 0xbffd, 0xffff4000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.19920115082382684342e-308, 0x3c00, 0xfd05f5efL, 0x1af543f5L), 0 },
	{ __LINE__,  HEXCONSTE(4.99994754791259765625e-01, 0x3ffd, 0xffff5000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.20134568199339657351e-308, 0x3c00, 0xfd451fdaL, 0x34ffb00eL), 0 },
	{ __LINE__,  HEXCONSTE(-4.99995231628417968750e-01, 0xbffd, 0xffff6000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.20349230233499970498e-308, 0x3c00, 0xfd845985L, 0xf34e8d48L), 0 },
	{ __LINE__,  HEXCONSTE(4.99995708465576171875e-01, 0x3ffd, 0xffff7000L, 0x00000000L), HEXCONSTE(1.02200000000000000000e+03, 0x4008, 0xff800000L, 0x00000000L), HEXCONSTE(2.20564101388188286058e-308, 0x3c00, 0xfdc3a2f6L, 0x429a6b50L), 0 },
	{ __LINE__,  HEXCONSTE(4.99992847442626953125e-01, 0x3ffd, 0xffff1000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.65971413147155051021e-4932, 0x0000, 0x654251daL, 0x326ef752L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99993324279785156250e-01, 0xbffd, 0xffff2000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.70159396531411688133e-4932, 0x0000, 0x66da7e09L, 0xfb94e936L), 0 },
	{ __LINE__,  HEXCONSTE(4.99993801116943359375e-01, 0x3ffd, 0xffff3000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.74413319774186890145e-4932, 0x0000, 0x68791774L, 0xe63198c4L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99994277954101562500e-01, 0xbffd, 0xffff4000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.78734221035123519122e-4932, 0x0000, 0x6a1e3802L, 0x01b2c62cL), 0 },
	{ __LINE__,  HEXCONSTE(4.99994754791259765625e-01, 0x3ffd, 0xffff5000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.83123154817675836303e-4932, 0x0000, 0x6bc9fa00L, 0xc22e6452L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99995231628417968750e-01, 0xbffd, 0xffff6000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.87581192226395178357e-4932, 0x0000, 0x6d7c782aL, 0xa516444cL), 0 },
	{ __LINE__,  HEXCONSTE(4.99995708465576171875e-01, 0x3ffd, 0xffff7000L, 0x00000000L), HEXCONSTE(1.63820000000000000000e+04, 0x400c, 0xfff80000L, 0x00000000L), HEXCONSTE(2.92109421228265595610e-4932, 0x0000, 0x6f35cda4L, 0xdc8b10d2L), 0 },
	{ __LINE__,  HEXCONSTE(4.99992847442626953125e-01, 0x3ffd, 0xffff1000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.32983804197785400131e-4932, 0x0000, 0x32a0f976L, 0x02d93405L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99993324279785156250e-01, 0xbffd, 0xffff2000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(-1.35077894757161189338e-4932, 0x8000, 0x336d1205L, 0x66a6168aL), 0 },
	{ __LINE__,  HEXCONSTE(4.99993801116943359375e-01, 0x3ffd, 0xffff3000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.37204958831014980365e-4932, 0x0000, 0x343c6149L, 0x41914edeL), 0 },
	{ __LINE__,  HEXCONSTE(-4.99994277954101562500e-01, 0xbffd, 0xffff4000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(-1.39365515587555531326e-4932, 0x8000, 0x350ef435L, 0xabd8a272L), 0 },
	{ __LINE__,  HEXCONSTE(4.99994754791259765625e-01, 0x3ffd, 0xffff5000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.41560092368791705746e-4932, 0x0000, 0x35e4d7f2L, 0xf326ef69L), 0 },
	{ __LINE__,  HEXCONSTE(-4.99995231628417968750e-01, 0xbffd, 0xffff6000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(-1.43789224819213050168e-4932, 0x8000, 0x36be19deL, 0x6cfdce8fL), 0 },
	{ __LINE__,  HEXCONSTE(4.99995708465576171875e-01, 0x3ffd, 0xffff7000L, 0x00000000L), HEXCONSTE(1.63830000000000000000e+04, 0x400c, 0xfffc0000L, 0x00000000L), HEXCONSTE(1.46053457016496072197e-4932, 0x0000, 0x379ac78bL, 0x4c6f2a62L), 0 },
#endif
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(2.00017070770263671875e+00, 0x4000, 0x8002cc00L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.03920942589966866653e-308, 0x3c00, 0xea9da8e3L, 0x3d761ff3L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00008416175842285156e+00, 0x4000, 0x80016100L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.13141132926681923956e-308, 0x3c00, 0xf53951d5L, 0x0228fdbaL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00008392333984375000e+00, 0x4000, 0x80016000L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.13167100805201266221e-308, 0x3c00, 0xf540f7d4L, 0x23ee0657L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00008404254913330078e+00, 0x4000, 0x80016080L, 0x00000000L), HEXCONSTE(-1.02200000000000000000e+03, 0xc008, 0xff800000L, 0x00000000L), HEXCONSTE(2.13154116470106799502e-308, 0x3c00, 0xf53d24ccL, 0xef0aeabbL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02231713867187500000e+03, 0xc008, 0xff944c00L, 0x00000000L), HEXCONSTE(1.78597830235888235660e-308, 0x3c00, 0xcd7b241bL, 0xc8bef000L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(-1.02230664062500000000e+03, 0xc008, 0xff93a000L, 0x00000000L), HEXCONSTE(1.79902171454620233154e-308, 0x3c00, 0xcefb505eL, 0x7e0af001L), FLAG_INEXACT },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e-01, 0x3ffb, 0xccccccccL, 0xcccccccdL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999980e-03, 0x3ff8, 0xa3d70a3dL, 0x70a3d70aL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999958e-04, 0x3ff5, 0x83126e97L, 0x8d4fdf3bL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000001e-04, 0x3ff1, 0xd1b71758L, 0xe219652cL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999995e-06, 0x3fee, 0xa7c5ac47L, 0x1b478423L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000004e-06, 0x3feb, 0x8637bd05L, 0xaf6c69b6L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+01, 0x4002, 0xa0000000L, 0x00000000L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999985e-08, 0x3fe7, 0xd6bf94d5L, 0xe57a42bcL), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999880790714001932e-01, 0x3ffe, 0xfffffe00L, 0x00010000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999777955e-01, 0x3ffe, 0xffffffffL, 0xfffff000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999892e-01, 0x3ffe, 0xffffffffL, 0xfffffffeL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999821186076331969e-01, 0x3ffe, 0xfffffd00L, 0x00030000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999666933e-01, 0x3ffe, 0xffffffffL, 0xffffe800L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999837e-01, 0x3ffe, 0xffffffffL, 0xfffffffdL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999761581442214720e-01, 0x3ffe, 0xfffffc00L, 0x00060000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999555911e-01, 0x3ffe, 0xffffffffL, 0xffffe000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999783e-01, 0x3ffe, 0xffffffffL, 0xfffffffcL), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999701976811650184e-01, 0x3ffe, 0xfffffb00L, 0x000a0000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999999999999444888e-01, 0x3ffe, 0xffffffffL, 0xffffd800L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999729e-01, 0x3ffe, 0xffffffffL, 0xfffffffbL), FLAG_INEXACT },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999642372184638361e-01, 0x3ffe, 0xfffffa00L, 0x000f0000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999333866e-01, 0x3ffe, 0xffffffffL, 0xffffd000L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999675e-01, 0x3ffe, 0xffffffffL, 0xfffffffaL), FLAG_INEXACT },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999582767561179253e-01, 0x3ffe, 0xfffff900L, 0x00150000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999999999999222844e-01, 0x3ffe, 0xffffffffL, 0xffffc800L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999621e-01, 0x3ffe, 0xffffffffL, 0xfffffff9L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000005960464832810e+00, 0x3fff, 0x80000080L, 0x00008000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000011102e+00, 0x3fff, 0x80000000L, 0x00000400L), FLAG_INEXACT },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), FLAG_INEXACT },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000011920930020892e+00, 0x3fff, 0x80000100L, 0x00018000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), 0 },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), FLAG_INEXACT },
#endif
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000017881395564245e+00, 0x3fff, 0x80000180L, 0x00030000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000033307e+00, 0x3fff, 0x80000000L, 0x00000c00L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000022e+00, 0x3fff, 0x80000000L, 0x00000002L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000023841861462870e+00, 0x3fff, 0x80000200L, 0x00050000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000044409e+00, 0x3fff, 0x80000000L, 0x00001000L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000022e+00, 0x3fff, 0x80000000L, 0x00000002L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000029802327716766e+00, 0x3fff, 0x80000280L, 0x00078000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000055511e+00, 0x3fff, 0x80000000L, 0x00001400L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000033e+00, 0x3fff, 0x80000000L, 0x00000003L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000035762794325933e+00, 0x3fff, 0x80000300L, 0x000a8000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000066613e+00, 0x3fff, 0x80000000L, 0x00001800L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000033e+00, 0x3fff, 0x80000000L, 0x00000003L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999940395355224609e-01, 0x3ffe, 0xffffff00L, 0x00000000L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000041723261290372e+00, 0x3fff, 0x80000380L, 0x000e0000L), 0 },
	{ __LINE__,  HEXCONSTE(9.99999999999999888978e-01, 0x3ffe, 0xffffffffL, 0xfffff800L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000077716e+00, 0x3fff, 0x80000000L, 0x00001c00L), FLAG_INEXACT },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(9.99999999999999999946e-01, 0x3ffe, 0xffffffffL, 0xffffffffL), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000043e+00, 0x3fff, 0x80000000L, 0x00000004L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000023841859331242e+00, 0x3fff, 0x80000200L, 0x00020000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000044409e+00, 0x3fff, 0x80000000L, 0x00001000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(2.00000000000000000000e+00, 0x4000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000022e+00, 0x3fff, 0x80000000L, 0x00000002L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000035762791128491e+00, 0x3fff, 0x80000300L, 0x00060000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000066613e+00, 0x3fff, 0x80000000L, 0x00001800L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(3.00000000000000000000e+00, 0x4000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000033e+00, 0x3fff, 0x80000000L, 0x00000003L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000047683724346825e+00, 0x3fff, 0x80000400L, 0x000c0000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000088818e+00, 0x3fff, 0x80000000L, 0x00002000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(4.00000000000000000000e+00, 0x4001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000043e+00, 0x3fff, 0x80000000L, 0x00000004L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000059604658986245e+00, 0x3fff, 0x80000500L, 0x00140000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000111022e+00, 0x3fff, 0x80000000L, 0x00002800L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(5.00000000000000000000e+00, 0x4001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000054e+00, 0x3fff, 0x80000000L, 0x00000005L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000071525595046751e+00, 0x3fff, 0x80000600L, 0x001e0000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000133227e+00, 0x3fff, 0x80000000L, 0x00003000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(6.00000000000000000000e+00, 0x4001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000065e+00, 0x3fff, 0x80000000L, 0x00000006L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000083446532528353e+00, 0x3fff, 0x80000700L, 0x002a0001L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000155431e+00, 0x3fff, 0x80000000L, 0x00003800L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(7.00000000000000000000e+00, 0x4001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000076e+00, 0x3fff, 0x80000000L, 0x00000007L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999880790724660073e-01, 0x3ffe, 0xfffffe00L, 0x00040000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999777955e-01, 0x3ffe, 0xffffffffL, 0xfffff000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-1.00000000000000000000e+00, 0xbfff, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999892e-01, 0x3ffe, 0xffffffffL, 0xfffffffeL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999761581463531002e-01, 0x3ffe, 0xfffffc00L, 0x000c0000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999555911e-01, 0x3ffe, 0xffffffffL, 0xffffe000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-2.00000000000000000000e+00, 0xc000, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999783e-01, 0x3ffe, 0xffffffffL, 0xfffffffcL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999642372216612785e-01, 0x3ffe, 0xfffffa00L, 0x00180000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999333866e-01, 0x3ffe, 0xffffffffL, 0xffffd000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-3.00000000000000000000e+00, 0xc000, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999675e-01, 0x3ffe, 0xffffffffL, 0xfffffffaL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999523162983905368e-01, 0x3ffe, 0xfffff800L, 0x0027ffffL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999111822e-01, 0x3ffe, 0xffffffffL, 0xffffc000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-4.00000000000000000000e+00, 0xc001, 0x80000000L, 0x00000000L), HEXCONSTE(9.99999999999999999566e-01, 0x3ffe, 0xffffffffL, 0xfffffff8L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999403953765408860e-01, 0x3ffe, 0xfffff600L, 0x003bffffL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999999999998889777e-01, 0x3ffe, 0xffffffffL, 0xffffb000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-5.00000000000000000000e+00, 0xc001, 0xa0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999458e-01, 0x3ffe, 0xffffffffL, 0xfffffff6L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999284744561123153e-01, 0x3ffe, 0xfffff400L, 0x0053fffeL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999998667732e-01, 0x3ffe, 0xffffffffL, 0xffffa000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-6.00000000000000000000e+00, 0xc001, 0xc0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999349e-01, 0x3ffe, 0xffffffffL, 0xfffffff4L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000011920928955078e+00, 0x3fff, 0x80000100L, 0x00000000L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999165535371048301e-01, 0x3ffe, 0xfffff200L, 0x006ffffdL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(1.00000000000000000000e+00, 0x3fff, 0x80000000L, 0x00000000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000022204e+00, 0x3fff, 0x80000000L, 0x00000800L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999999999998445688e-01, 0x3ffe, 0xffffffffL, 0xffff9000L), 0 },
	{ __LINE__,  HEXCONSTE(1.00000000000000000011e+00, 0x3fff, 0x80000000L, 0x00000001L), HEXCONSTE(-7.00000000000000000000e+00, 0xc001, 0xe0000000L, 0x00000000L), HEXCONSTE(9.99999999999999999241e-01, 0x3ffe, 0xffffffffL, 0xfffffff2L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.06250000000000000000e+00, 0x3fff, 0x88000000L, 0x00000000L), HEXCONSTE(1.12500000000000000000e+00, 0x3fff, 0x90000000L, 0x00000000L), HEXCONSTE(1.07058229302876136216e+00, 0x3fff, 0x8908d730L, 0x1e1ba6a0L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.50000000000000000000e+00, 0x3fff, 0xc0000000L, 0x00000000L), HEXCONSTE(1.03125000000000000000e+00, 0x3fff, 0x84000000L, 0x00000000L), HEXCONSTE(1.51912709871474318406e+00, 0x3fff, 0xc272c1bbL, 0xb93d60c6L), 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(1.50000000000000000000e+00, 0x3fff, 0xc0000000L, 0x00000000L), HEXCONSTE(6.27710117417006039155e+57, 0x40be, 0xfffffe80L, 0x00006000L), 0 },
	{ __LINE__,  HEXCONSTE(1.49357829132402765485e+205, 0x42a8, 0xbe8d0537L, 0x90000000L), HEXCONSTE(1.50000000000000000000e+00, 0x3fff, 0xc0000000L, 0x00000000L), HEXCONSTE(5.77220822056602682675e+307, 0x43fd, 0xa465dfdcL, 0x82053400L), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.80653297901153564453e+00, 0x3fff, 0xe73c7900L, 0x00000000L), HEXCONSTE(-3.99451828002929687500e+01, 0xc004, 0x9fc7de00L, 0x00000000L), HEXCONSTE(5.49849462130295096918e-11, 0x3fdc, 0xf1d38bbaL, 0xcb34009aL), 0 },
	{ __LINE__,  HEXCONSTE(3.40282346638528859812e+38, 0x407e, 0xffffff00L, 0x00000000L), HEXCONSTE(-5.70981264114379882812e+00, 0xc001, 0xb6b6c900L, 0x00000000L), HEXCONSTE(9.78174187501496214280e-221, 0x3d24, 0x8d6f08ffL, 0x7f0c2c0eL), FLAG_INEXACT },
	{ __LINE__,  HEXCONSTE(1.26192164421081542969e+00, 0x3fff, 0xa186a600L, 0x00000000L), HEXCONSTE(8.08921203613281250000e+01, 0x4005, 0xa1c8c400L, 0x00000000L), HEXCONSTE(1.48841555176933137220e+08, 0x401a, 0x8df24532L, 0xd4b7d75fL), FLAG_INEXACT },
#if defined(TEST_LDOUBLE) && LDBL_MAX_EXP > 1024
	{ __LINE__,  HEXCONSTE(5.96560120582580566406e-01, 0x3ffe, 0x98b82a00L, 0x00000000L), HEXCONSTE(-6.55302978515625000000e+03, 0xc00b, 0xccc83d00L, 0x00000000L), HEXCONSTE(1.39505874306152339711e+1470, 0x5312, 0xd20e27f1L, 0x91eac857L), 0 },
#endif
	{ __LINE__,  HEXCONSTE(8.40579330921173095703e-01, 0x3ffe, 0xd7303500L, 0x00000000L), HEXCONSTE(-3.27732421875000000000e+02, 0xc007, 0xa3ddc000L, 0x00000000L), HEXCONSTE(5.22399857785103091761e+24, 0x4051, 0x8a4730f8L, 0xb21a6325L), 0 },
	{ __LINE__,  HEXCONSTE(1.02191865444183349609e+00, 0x3fff, 0x82ce3b00L, 0x00000000L), HEXCONSTE(4.09202319335937500000e+03, 0x400a, 0xffc05f00L, 0x00000000L), HEXCONSTE(3.40273710581749986659e+38, 0x407e, 0xfffe5535L, 0xa38f9be6L), 0 },
};

int main(int argc, char **argv)
{
	int status;
	
	status = 0;
	
	test_init(argc, argv);
	
	status |= test_table_ff_f(fpow_data, ARRAY_SIZE(fpow_data), __FILE__);
	
	return status ? EXIT_FAILURE : EXIT_SUCCESS;
}
